<?php
namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;

class TipoAvaliacaoController extends Crud {
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRTIPOAVALIACAO');
    protected $tipoAvaliacaoService;
    
    protected $dataSourceNameTipo = 'avl_tipoavaliacao';
    protected $dataSourceNameGetTipo = 'getTipoAvaliacao';
    protected $dataSourceNameQuestao = 'avl_tipoavaliacaoquestao';
    protected $dataSourceNameGetQuestao = 'getTipoAvaliacaoQuestao';

    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entitymanager, \HCM\AVL\Services\tipoAvaliacaoService $tipoAvaliacaoService){
        $this->parameterBag = $parameterBag;
        $this->entitymanager = $entitymanager;
        $this->tipoAvaliacaoService = $tipoAvaliacaoService;
        parent::__construct($dataSourceManager);
    }
    public function getTipoAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetTipo);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function deleteTipoAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
            $this->dataSourceName = $this->dataSourceNameTipo;
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $questoes = $this->tipoAvaliacaoService->getQuestoesTipoAvaliacao($dataset);
            if($questoes === false){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
                parent::delete($request, $response);
            }else{
                $response->addDataSet(new DataSet('questoes', $questoes));
            }
          
    }
    
    public function setTipoAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameTipo;
            foreach($request->getDataSet()->getRows() as $row) {
            if (array_key_exists('NRTIPOAVALIACAO', $row)) {
                    if ($row['NRTIPOAVALIACAO'] === '') {
                        $row['NRTIPOAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPOAVALIACAO', 1, 22, true);
                    }
                } else {
                        $row['NRTIPOAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPOAVALIACAO', 1, 22, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function getTipoAvaliacaoQuestao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetQuestao);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    public function deleteTipoAvaliacaoQuestao(DTO\Request\DataSet $request, DTO\Response $response){
       try{
            $this->dataSourceName = $this->dataSourceNameQuestao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        
    }
    
    public function setTipoAvaliacaoQuestao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameQuestao;
            foreach($request->getDataSet()->getRows() as $row) {
                if (array_key_exists('NRQUESTIPOAVAL', $row)) {
                        if ($row['NRQUESTIPOAVAL'] === '') {
                            $row['NRQUESTIPOAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTIPOAVAL', 1, 22, true);
                            if(!$this->tipoAvaliacaoService->validaQuestaoTipoAvaliacao($row)){
                                $response->addMessage(new Message('A questão já esta cadastrada para este tipo de avaliação', Message::TYPE_WARNING));
                                return false;
                            }
                        }
                    } else {
                        $row['NRQUESTIPOAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTIPOAVAL', 1, 22, true);
                        if(!$this->tipoAvaliacaoService->validaQuestaoTipoAvaliacao($row)){
                            $response->addMessage(new Message('A questão já esta cadastrada para este tipo de avaliação', Message::TYPE_WARNING));
                            return false;
                        }

                }
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}