<?php

namespace HCM\FPA\ObrigacaoLegal\SEFIP;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\MsgCalculo;
use HCM\Util\Valor;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculoempregEntity;
use HCM\Geral\Entity\GpeOcupacaomEntity;
use HCM\Geral\Entity\GpeCboEntity;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\Geral\Entity\GpealtesalarioEntity;
use HCM\Geral\Entity\GpetpmovtransfmEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaitecalcestrutEntity;
use HCM\FPA\Entity\FpaassoceventoEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpagfipmagnetEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\FPA\Registro\RegistroSEFIP;
use HCM\FPA\Registro\sefip\Registro10SEFIP;
use HCM\FPA\Registro\sefip\Registro12SEFIP;
use HCM\FPA\Registro\sefip\Registro20SEFIP;
use HCM\FPA\Registro\sefip\Registro21SEFIP;
use HCM\FPA\Registro\sefip\Registro30SEFIP;
use HCM\FPA\Registro\sefip\Registro32SEFIP;

class SEFIP
{

    CONST ALFANUMERICOS = '/[^a-zA-Z0-9]/';
    CONST ESTRUTURALEGAL = 'ESTRUTURALEGAL';
    CONST ESTRUTURAGERENCIAL = 'ESTRUTURAGERENCIAL';
    CONST TRANSFERENCIA_ENTRADA = 8;
    CONST TRANSFERENCIA_SAIDA = 9;
	CONST CODFGTS_SITUFUNC_LICENCAMATERNIDADE = 'Q1';
	CONST CODFGTS_SITUFUNC_PRORROGACAOLICENCAMATERNIDADE = 'Q2';
	CONST NRSITUFUNC_LICENCAMATERNIDADE = 5;
    CONST CODFGTS_RESCISAO_INICIAT_EMPREGADO = 'J';
    CONST CODFGTS_RESCISAO_COMPLEMENTAR = 'V3';
    CONST CODFGTS_RESCISAO_JUSTA_CAUSA_EMPREGADOR = 'H';

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_movimentacao;
    private $_estruturacao;
    private $_tpMovTransfM;
    private $_hierarquiaOrg;
    private $_nrCategoriaTrabalhador;
    private $_registro;
    private $_registro10;
    private $_registro12;
    private $_registro20;
    private $_registro21;
    private $_registro30;
    private $_registro32;
    private $_gfipMagnetico;
    private $_gfipMagneticoTrab;
    private $_dadoAlterado = array(array());
    private $_dadoVinculo = array();
    private $_dadoEstrutura = array();
    private $_vrSalarioFamilia;
    private $_vrSalarioMaternidade;
    private $_nrInscricao;
    private $_nrInscricaoTomador;
    private $_nrTpInscricao;
    private $_nrTpInscricaoTomador;
    private $_codMovimentacao;
    private $_pessoa;
    private $_sitFuncional;
    private $_vinculoEmpreg;
    private $_ocupacao;
    private $_cbo;
    private $_altSitFuncional;
    private $_alteracaoSalario;
    private $_tpDemissao;
    private $_listaCalculoFolha;
    private $_itemCalculo;
    private $_itemCalcEstrut;
    private $_assocEvento;
    private $_evento;
    private $_itemGrupo;
    private $_nrPessoaResp;
    private $_vinculom;
    private $_cdOperador;
    private $_competencia13;
    private $_valorReferenciam;


    public function __construct($nrOrg, $nrOrgPadrao, $mesCompetencia, $gfipMagnetico,
                                $dataSet, $competencia13, $itensGrupo, $listaNrCalculoFolha, $entityManager, $hcmParameters)
    {

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;
        $this->_dtInicio = $this->_mesCompetencia;
        $this->_dtFim = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_sitFuncional = new GpesitufuncmEntity($entityManager, $hcmParameters);
        $this->_vinculoEmpreg = new GpevinculoempregEntity($entityManager, $hcmParameters);
        $this->_ocupacao = new GpeOcupacaomEntity($entityManager, $hcmParameters);
        $this->_cbo = new GpeCboEntity($entityManager, $hcmParameters);
        $this->_altSitFuncional = new GpealtesitufuncEntity($entityManager, $hcmParameters);
        $this->_alteracaoSalario = new GpealtesalarioEntity($entityManager, $hcmParameters);
        $this->_tpDemissao = new FpaTpdemissaoEntity($entityManager, $hcmParameters);
        $this->_tpMovTransfM = new GpetpmovtransfmEntity($entityManager, $hcmParameters);
        $this->_listaCalculoFolha = $listaNrCalculoFolha;
        $this->_itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_itemCalcEstrut = new FpaitecalcestrutEntity($entityManager, $hcmParameters);
        $this->_assocEvento = new FpaassoceventoEntity($entityManager, $hcmParameters);
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        $this->_itemGrupo = $itensGrupo;
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_gfipMagnetico = $gfipMagnetico;
        $this->_gfipMagneticoTrab = new FpagfipmagnetEntity($entityManager, $hcmParameters);
        $this->_estruturam = new EstruturamEntity($entityManager, $hcmParameters);
        $this->_registro10 = new Registro10SEFIP();
        $this->_registro12 = new Registro12SEFIP();
        $this->_registro20 = new Registro20SEFIP();
        $this->_registro21 = new Registro21SEFIP();
        $this->_registro30 = new Registro30SEFIP();
        $this->_registro32 = new Registro32SEFIP();
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_cdOperador = FPA::getCdOperador();
        $this->_competencia13 = $competencia13;
        $this->_valorReferenciam = new FpaValorreferenciamEntity($entityManager, $hcmParameters);
        $this->_inicializaVariaveis($dataSet, $competencia13);

    }

    private function _inicializaVariaveis($dataSet, $competencia13)
    {
        try {
            if ($dataSet['NRHIERARQORG'] !== '') {
                $this->_estruturacao->carregaHierarquias($dataSet['NRHIERARQORG']);
            } else {
                throw new \Exception("Favor Informar a Hierarquia.");
            }
        } catch(\Exception $e) {
            throw new \Exception("Hierarquia da Organização: ".$e->getMessage());
        }
        try {
            $this->_estruturacao->carregaHierarquiasTomador($dataSet['NRHIERARTOMADOR']);
        } catch(\Exception $e) {
            throw new \Exception("Hierarquia do Tomador: ".$e->getMessage());
        }
        $this->_estruturacao->carregaTipoHierarquia($dataSet['NRHIERARQORG']);
        $this->_estruturacao->carregaTipoHierarquiaTomador($dataSet['NRHIERARTOMADOR']);
        $this->_registro10->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro10->setCodCentralizadorFGTS($dataSet['IDCENTRALFGTS']);
        $this->_registro10->setCNPJCentralizador($dataSet['NRINSCCENTRALIZ']);
        $this->_registro10->setCodPagtoGPS($dataSet['CODPAGTOGPS']);
        $this->_registro10->setCompetencia13($competencia13);
        $this->_registro12->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro12->setIdRecolhimentoGPS($dataSet['IDRECOLHGPS']);
        $this->_registro12->setCompetencia($this->_mesCompetencia);
        $this->_registro12->setCompetencia13($competencia13);
        $this->_registro12->setProcesso($dataSet['NRPROCESSO']);
        $this->_registro12->setProcessoAno($dataSet['NRANOPROCESSO']);
        $this->_registro12->setVaraJCJ($dataSet['NRVARAJCJ']);
        $this->_registro12->setPerInicio(substr($dataSet['DTINIPERIODO'],2,4).substr($dataSet['DTINIPERIODO'],0,2));
        $this->_registro12->setPerFim(substr($dataSet['DTFIMPERIODO'],2,4).substr($dataSet['DTFIMPERIODO'],0,2));
        $this->_registro20->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro20->setCodPagtoGPS($dataSet['CODPAGTOGPS']);
        $this->_registro20->setCompetencia13($competencia13);
        $this->_registro21->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro21->setIdRecolhimentoGPS($dataSet['IDRECOLHGPS']);
        $this->_registro21->setCodPagtoGPS($dataSet['CODPAGTOGPS']);
        $this->_registro21->setCompetencia13($competencia13);
        $this->_registro30->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro30->setCompetencia($this->_mesCompetencia);
        $this->_registro30->setCompetencia13($competencia13);
        $this->_registro32->setCodigoRecolhimento($dataSet['NRRECOLHIMENTO']);
        $this->_registro32->setCompetencia($this->_mesCompetencia);
        $this->_registro32->setCompetencia13($competencia13);
        $this->_nrPessoaResp = $dataSet['NRPESSOARESPONS'];
    }

    public function geraSEFIP($vinculo, $nrTipoHierarquiaOrg, $nrTipoHierarquiaTomador, $tpInscHierarq, $tpInscHierarqTom)
    {
        $this->_calculaBasesSEFIP($vinculo->getNrvinculom());
        $this->_vinculo = $vinculo;
        $this->_dtRescisao = $this->_vinculo->getDtrescisaovinc();
        $this->_codMovimentacao = null;
        
        $ultimoDiaCompetencia = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        
        
        $movimentacoes = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacoes($this->_nrOrg, FPA::getTipoEstruturaLegalOrg(),
            $vinculo->getNrvinculom());
        
        if ($movimentacoes) {
            foreach ($movimentacoes as $movimentacao) {
                
                $estrutura = $this->_estruturam->retornaEstrutura($movimentacao->getNrestruturam());
                
                $nrCei  = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdceiestrut());
                $nrCnpj = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdcnpjestrut());

                if (!empty($nrCei)) {
                    $this->_nrInscricao = $nrCei;
                    $this->_nrTpInscricao = RegistroSEFIP::CEI;
                } else {
                    $this->_nrInscricao = $nrCnpj;
                    $this->_nrTpInscricao = RegistroSEFIP::CNPJ;
                }

                $this->_preparaRegistro10($estrutura, $vinculo->getNrvinculom());

                $this->_preparaRegistro12($estrutura);

                $ocupacao = $this->_ocupacao->retornaOcupacao($vinculo->historico->getNrcargo());

                $objTpDemissao = $this->_tpDemissao->procuraTipoDemissao($vinculo->getNrtpdemissao());
                if (is_object($objTpDemissao) && $this->_dtRescisao) {
                    $codFgts = $objTpDemissao->getCodfgts();
                    if (($codFgts == self::CODFGTS_RESCISAO_INICIAT_EMPREGADO || $codFgts == self::CODFGTS_RESCISAO_JUSTA_CAUSA_EMPREGADOR) && !($ultimoDiaCompetencia < $this->_dtRescisao)) {
                        if ($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_SEM_13SAL] == 0) {
                            $this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_SEM_13SAL] = 0.01;
                        }
						if ($codFgts == self::CODFGTS_RESCISAO_INICIAT_EMPREGADO) {
							if ($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_13SAL_MOV] == 0) {
								$this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_13SAL_MOV] = 0.01;
							}
						}	
                    }
                }

                if (((!$this->_competencia13) && ($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_SEM_13SAL] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_13SAL_PAGO_MES] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA]))
                    or
                    (($this->_competencia13) && ($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_13SAL_MOV_13]))) {
                    if ($tpInscHierarqTom) {
                        
                        $movimentacaoGeren =  $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacao(
                            $this->_nrOrg, FPA::getTipoEstruturaGerencialOrg(), $vinculo->getNrvinculom()
                        );
                        
                        if ($movimentacaoGeren[0]) {
                            $estruturaGeren = $this->_estruturam->retornaEstrutura($movimentacaoGeren[0]->getNrestruturam());
                            if ($estruturaGeren) {
                                $estruturaTomador = $this->_estruturam->getEstruturaSuperior($nrTipoHierarquiaTomador, $estruturaGeren);
                                if ($estruturaTomador) {
                                    $pessoa = $this->_pessoa->retornaPessoaAtual($vinculo->getNrpessoa());
                                    $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();
                                    if (!$nrPisPasepCi) {
                                        MsgCalculo::getInstance()->adicionaMensagem('Pessoa ' . $pessoa->historico->getNmpessoa() . ' não possui PIS/PASEP.');
                                    }
                                    if ($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA] && !($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_SEM_13SAL] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_13SAL_PAGO_MES] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_13SAL_MOV_13])) {
                                        $this->_preparaRegistro20($estruturaTomador, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                    } else {
                                        $this->_preparaRegistro20($estruturaTomador, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                        $this->_preparaRegistro21($estruturaTomador, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                        $this->_preparaRegistro30($estruturaGeren->getNrtipoestrutura(), $vinculo, $ocupacao, $estruturaTomador);
                                    }
                                } else {
                                    if ($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA] && !($this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_SEM_13SAL] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_13SAL_PAGO_MES] || $this->_dadoVinculo[RegistroSEFIP::SEFIP_REM_13SAL_MOV_13])) {
                                        $this->_preparaRegistro20($estruturaTomador, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                    } else {
                                        $this->_preparaRegistro20($estrutura, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                        $this->_preparaRegistro21($estrutura, $tpInscHierarqTom, $vinculo->getNrvinculom());
                                        $this->_preparaRegistro30($estruturaGeren->getNrtipoestrutura(), $vinculo, $ocupacao, $estruturaTomador);
                                    }
                                }

                            } else {
                                MsgCalculo::getInstance()->adicionaMensagem('Movimentação do vínculo ' . $vinculo->getNmvinculom() . ' não possui estrutura.');
                           }

                        }
                    } else {
                        $this->_preparaRegistro30($movimentacao->getNrtipoestrutura(), $vinculo, $ocupacao, $movimentacao, null);
                    }

                    if ($this->_competencia13 === false) {
                        $this->_preparaRegistro32($vinculo, $movimentacao, $estruturaTomador);
                    }

                } else {
                    if (!$this->_competencia13) {
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $vinculo->getNrvinculom() . ' não possui FGTS calculado.');
                    } else {
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $vinculo->getNrvinculom() . ' não possui 13º calculado.');
                    }
                }
            }
        } else {
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura nao encontrada, com data de vigencia maior do que a data de competencia ou nao definido em competencia ou nao definido em hierarquia para o vinculo ' . $vinculo->getNrvinculom(), MsgCalculo::ERRO);
        }
    }

    private function _preparaRegistro10($estrutura, $nrVinculo)
    {
        $posicao = $this->_gfipMagnetico->procuraRegistroLista(
            $this->_nrInscricao, "0", Registro10SEFIP::REGISTRO_10, $this->_nrOrg ,$this->_cdOperador
        );
        
        if ($posicao === false) {
            $endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $this->_registro10->setTpInscricao($this->_nrTpInscricao);
            $this->_registro10->setNrInscricao($this->_nrInscricao);
            $this->_registro10->geraRegistro10($estrutura, $endereco);
            $this->_gfipMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro10SEFIP::REGISTRO_10, $this->_nrTpInscricao,
                $this->_nrInscricao, $this->_registro10->getLinhaRegistro(), $this->_cdOperador
            );
            $this->_registro10->setLinhaRegistro();
        }
        $this->_consolidaDadosSEFIP(Registro10SEFIP::REGISTRO_10, $this->_nrInscricao, 0);
    }

    private function _preparaRegistro12($estrutura)
    {
        $posicao = $this->_gfipMagnetico->procuraRegistroLista(
            $this->_nrInscricao, "0", Registro12SEFIP::REGISTRO_12, $this->_nrOrg, $this->_cdOperador
        );

        if ($posicao === false) {

            $this->_calculaBasesEstrutSEFIP($estrutura->getNrestruturam());

            $this->_registro12->setTpInscricao($this->_nrTpInscricao);
            $this->_registro12->setNrInscricao($this->_nrInscricao);
            $this->_registro12->setValoresEstrutura($this->_dadoEstrutura);

            $this->_registro12->geraRegistro12($estrutura, $this->_dadoVinculo);
            $this->_gfipMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro12SEFIP::REGISTRO_12, $this->_nrTpInscricao,
                $this->_nrInscricao, $this->_registro12->getLinhaRegistro(), $this->_cdOperador
            );
            $this->_registro12->setLinhaRegistro();
        }
        $this->_consolidaDadosSEFIP(Registro12SEFIP::REGISTRO_12, $this->_nrInscricao, 0);
    }

    private function _preparaRegistro20($movimentacao, $tpInscHierarq, $nrVinculo)
    {

        if ($movimentacao !== null) {
            $estrutura     = $this->_estruturam->retornaEstrutura($movimentacao->getNrestruturam());
            if ($estrutura->historico) {
                $nrCeiTomador  = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdceiestrut());
                $nrCnpjTomador = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdcnpjestrut());
    
                if (!empty($nrCeiTomador)) {
                    $this->_nrInscricaoTomador = $nrCeiTomador;
                    $this->_nrTpInscricaoTomador = RegistroSEFIP::CEI;
                } else {
                    $this->_nrInscricaoTomador = $nrCnpjTomador;
                    $this->_nrTpInscricaoTomador = RegistroSEFIP::CNPJ;
                }
    
                $posicao = $this->_gfipMagnetico->procuraRegistroLista(
                    $this->_nrInscricao, $this->_nrInscricaoTomador, Registro20SEFIP::REGISTRO_20, $this->_nrOrg, $this->_cdOperador
                );
    
                if ($posicao === false) {
                    $endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
                    
                    $this->_calculaBasesEstrutSEFIP($movimentacao->getNrestruturam());
    
                    $this->_registro20->setTpInscricao($this->_nrTpInscricao);
                    $this->_registro20->setNrInscricao($this->_nrInscricao);
                    $this->_registro20->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                    $this->_registro20->setNrInscricaoTomador($this->_nrInscricaoTomador);
                    $this->_registro20->setValoresEstrutura($this->_dadoEstrutura);
    
                    $this->_registro20->geraRegistro20($movimentacao, $endereco);
                    $this->_gfipMagnetico->insereRegistroLista(
                        $this->_nrOrg, $this->_mesCompetencia, Registro20SEFIP::REGISTRO_20, $this->_nrTpInscricao,
                        $this->_nrInscricao, $this->_registro20->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
                        $this->_nrInscricaoTomador, null, null, '0', null, null, null, $estrutura->getNmestruturam()
                    );
                    $this->_registro20->setLinhaRegistro();
                }
    
                $this->_consolidaDadosSEFIP(Registro20SEFIP::REGISTRO_20, $this->_nrInscricao, $this->_nrInscricaoTomador);
                return true;
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Não foi encontrado histórico para a estrutura ' . $movimentacao->getNrestruturam() . ' do vínculo ' . $nrVinculo);
                return false;
            }
        } else {
            return false;
        }
    }

    private function _preparaRegistro21($movimentacao, $tpInscHierarq, $nrVinculo)
    {
        if ($movimentacao !== null) {

            $estrutura = $this->_estruturam->retornaEstrutura($movimentacao->getNrestruturam());
            if ($estrutura->historico) {
                $nrCeiTomador  = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdceiestrut());
                $nrCnpjTomador = preg_replace(self::ALFANUMERICOS, '', $estrutura->historico->getCdcnpjestrut());
    
                if (!empty($nrCeiTomador)) {
                    $this->_nrInscricaoTomador = $nrCeiTomador;
                    $this->_nrTpInscricaoTomador = RegistroSEFIP::CEI;
                } else {
                    $this->_nrInscricaoTomador = $nrCnpjTomador;
                    $this->_nrTpInscricaoTomador = RegistroSEFIP::CNPJ;
                }
    
                $posicao = $this->_gfipMagnetico->procuraRegistroLista(
                    $this->_nrInscricao, $this->_nrInscricaoTomador, Registro21SEFIP::REGISTRO_21, $this->_nrOrg, $this->_cdOperador
                );
    
                if ($posicao === false) {
    
                    $this->_calculaBasesEstrutSEFIP($movimentacao->getNrestruturam());
                    if ($this->_dadoEstrutura[RegistroSEFIP::SEFIP_COMPEN_VLRCORR] > 0) {
                        $this->_registro21->setTpInscricao($this->_nrTpInscricao);
                        $this->_registro21->setNrInscricao($this->_nrInscricao);
                        $this->_registro21->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                        $this->_registro21->setNrInscricaoTomador($this->_nrInscricaoTomador);
                        $this->_registro21->setValoresEstrutura($this->_dadoEstrutura);
    
                        $this->_registro21->geraRegistro21($movimentacao);
                        $this->_gfipMagnetico->insereRegistroLista(
                            $this->_nrOrg, $this->_mesCompetencia, Registro21SEFIP::REGISTRO_21, $this->_nrTpInscricao,
                            $this->_nrInscricao, $this->_registro21->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
                            $this->_nrInscricaoTomador, null, null, '0', null, null, null, /*$estrutura->getNmrazsocestrut()*/ $estrutura->getNmestruturam()
                        );
                        $this->_registro21->setLinhaRegistro();
                    }
                }
    
                return true;
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Não foi encontrado histórico para a estrutura ' . $movimentacao->getNrestruturam() . ' do vínculo ' . $nrVinculo);
                return false;
            }
        } else {
            return false;
        }
    }

    private function _preparaRegistro30($tipoEstrutura, $vinculo, $ocupacao, $estrutura)
    {
        $agentenocivo = $this->entityManager->getRepository(Repositories::GPE_AGENTENOCIVO)->findOneBy(array('nragentenocivo' => $vinculo->historico->getNragentenocivo()));
        if (is_object($agentenocivo) && $agentenocivo->getCdagentenocivo()) {
            $vinculo->ocorrencia = $agentenocivo->getCdagentenocivo();
        } else {
            if ($vinculo->historico->getIdmultvinc() !== GpevinculohEntity::IDMULTVINC_UNICO) {
                $vinculo->ocorrencia = '05';
            } else {
                if ($this->_dadoVinculo[RegistroSEFIP::SEFIP_DEDUCAO_INSS_PATRONAL] > 0) {
                    $vinculo->ocorrencia = '05';
                } else {
                    $vinculo->ocorrencia = null;
                }
            }
        }

        if (is_object($ocupacao) === true) {
            if (is_object($ocupacao->historico) === true) {
				$cbo = $this->_cbo->retornaCbo($ocupacao->historico->getNrcbo());
				if ($cbo) {
					$vinculo->cbo = $cbo->getCdrais();
				} else {
					$vinculo->cbo = null;
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $vinculo->getNrvinculom() . ' não possui CBO.');
				}
			} else {
				$vinculo->cbo = null;
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $vinculo->getNrvinculom() . ' não possui CBO.');
			}
        } else {
			$vinculo->cbo = null;
            MsgCalculo::getInstance()->adicionaMensagem('Nao parametrizado ocupação para o vinculo ' . $vinculo->getNrvinculom());
        }
      
        $situacaoFuncional = $this->_sitFuncional->retornaSituacaoFuncional(
            $this->_nrOrgPadrao, $this->_nrOrg, $vinculo->historico->getNrsitufuncm()
        );

		if ($situacaoFuncional) {
			$this->_nrCategoriaTrabalhador = $situacaoFuncional->historico->getCodcategtrab();
			if (empty($this->_nrCategoriaTrabalhador) === true) {
				$vinculoEmpreg = $this->_vinculoEmpreg->retornaVinculoEmpregaticio($vinculo->getNrvinculoempreg());
				if (is_object($vinculoEmpreg) === true) {
					$this->_nrCategoriaTrabalhador = $vinculoEmpreg->getCodcategtrab();
				}
			}
		} else {
            MsgCalculo::getInstance()->adicionaMensagem('Situação Funcional não parametrizada para o vínculo ' . $vinculo->getNrvinculom());
        }	

        $alteracoes = $this->_altSitFuncional->retornaAlteSituFuncporVinculo($vinculo->getNrvinculom(), $this->_nrOrg);
        foreach ($alteracoes as $item) {
            $iniSituaFunc = DateUtil::getDataDeString($item->getDtinisitufunc());
            if ($item->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_NORMAL && $iniSituaFunc < $this->_dtInicio && $item->getDtfimsitufunc()) {
                $fimSituaFunc = DateUtil::getDataDeString($item->getDtfimsitufunc());
                
                if(DateUtil::isBetween($fimSituaFunc, $this->_dtInicio, $this->_dtFim)){
                    $situacaoFuncional = $this->_sitFuncional->retornaSituacaoFuncional($this->_nrOrgPadrao, $this->_nrOrg, $item->getNrsitufuncm());
                    $this->_codMovimentacao = $situacaoFuncional->historico->getCodfgtsret();
                }
            }
        }

        $pessoa = $this->_pessoa->retornaPessoaAtual($vinculo->getNrpessoa());
        $nrModalidade = $vinculo->historico->getNrtpmodalidsal();
        $this->_registro30->setNrInscricao($this->_nrInscricao);
        $this->_registro30->setNrInscricaoTomador($this->_nrInscricaoTomador);
        $this->_registro30->setTpInscricao($this->_nrTpInscricao);
        $this->_registro30->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
        $this->_registro30->setCodMovimentacao($this->_codMovimentacao);
        $this->_registro30->setNrCategoria($this->_nrCategoriaTrabalhador);
        
        $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();
        if (!$nrPisPasepCi) {
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $vinculo->getNmvinculom() . ' sem PIS/PASEP/CI.');
        }

        $this->_registro30->geraRegistro30($pessoa, $vinculo, $this->_dadoVinculo);
        $vrSalarioFamilia = Valor::arredonda($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA], Valor::TOTAL, -2, 5);
        $vrSalarioMaternidade = Valor::arredonda($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE], Valor::TOTAL, -2, 5);

        $this->_gfipMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro30SEFIP::REGISTRO_30, $this->_nrTpInscricao,
            $this->_nrInscricao, $this->_nrTpInscricaoTomador, $this->_nrInscricaoTomador,
            $this->_registro30->getLinhaRegistro(), $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(),
            $nrPisPasepCi, $vrSalarioFamilia, $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador
        );
//            limpa o registro 30.
        $this->_registro30->setLinhaRegistro();
    }

    private function _preparaRegistro32($vinculo, $gpemovimentacao, $estrutura)
    {

        $pessoa = $this->_pessoa->retornaPessoaAtual($vinculo->getNrpessoa());
        $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();
        $dataRescisao = $vinculo->getDtrescisaovinc();

        if (empty($dataRescisao) === false) {
            $dataRescisao = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());
        }
        if (empty($dataRescisao) === false
            && (DateUtil::isBetween($dataRescisao, $this->_dtInicio, FPA::getUltimoDiaMes()) ||
                ($dataRescisao < $this->_dtInicio && array_sum($this->_dadoVinculo) > 0))
        ) {
            $objTpDemissao = $this->_tpDemissao->procuraTipoDemissao($vinculo->getNrtpdemissao());
            if (!$objTpDemissao) {
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$vinculo->getNrvinculom().' não possui tipo de demissão informado.');
            } else {
                if ($dataRescisao < $this->_dtInicio) {
                    $idGrrfComplementar = $this->_valorReferenciam->getValorReferencia(FpaValorreferenciamEntity::GERA_GRRF_COMPLEMENTAR);
                    if ($idGrrfComplementar == 1) {
                        $codFGTS = $objTpDemissao->getCodfgts();
                    } else {
                        $codFGTS = self::CODFGTS_RESCISAO_COMPLEMENTAR;
                    }
                } else {
                    $codFGTS = $objTpDemissao->getCodfgts();
                }
            }
            $this->_registro32->setNrInscricao($this->_nrInscricao);
            $this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
            $this->_registro32->setTpInscricao($this->_nrTpInscricao);
            $this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
            $this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
            $this->_registro32->setCodMovimentacao($codFGTS);
            $this->_registro32->setDtMovimentacao($dataRescisao);
            $this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
            $vrSalarioFamilia = Valor::arredonda($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA], Valor::TOTAL, -2, 5);
            $vrSalarioMaternidade = Valor::arredonda($this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE], Valor::TOTAL, -2, 5);
            
            $this->_gfipMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao,
                $this->_nrInscricao, $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
                $this->_nrInscricaoTomador, $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(),
                $nrPisPasepCi, $vrSalarioFamilia, $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador
            );
            $this->_registro32->setLinhaRegistro();
        }

        $alteracoes = $this->_altSitFuncional->retornaAlteSituFuncporVinculo($vinculo->getNrvinculom(), $this->_nrOrg);
        foreach ($alteracoes as $item) {

            if ($item->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_DEMITIDO) {

                $gerouMovimento = false;
                $dtFim = $item->getDtfimsitufunc();
                $dtIni = $item->getDtinisitufunc();
                $iniSituaFunc = DateUtil::getDataDeString($dtIni);

                if ((empty($dtFim) === true) || ($this->_dtFim > $dtIni)) {
                    $testa_inicio = false;
                    if (empty($dtFim)) {
                        if ($iniSituaFunc > $this->_dtFim) {
                            $testa_inicio = false;
                        } else {
                            $testa_inicio = true;
                        }
                    } else {
                        if ($this->_dtInicio > $dtFim !== true) {
                            $testa_inicio = true;
                        }
                    }
                    if ($testa_inicio) {

                        if ($item->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_NORMAL) {
                            $situacaoFuncional = $this->_sitFuncional->retornaSituacaoFuncional($this->_nrOrgPadrao,
                                $this->_nrOrg, $item->getNrsitufuncm());
                            if(is_object($situacaoFuncional)){
                                $codFGTS = $situacaoFuncional->historico->getCodfgts();
                            }
                            if (empty($codFGTS) !== true) {
                                $dtinicial = DateUtil::subtraiIntervalo($iniSituaFunc, 1);
                                $this->_registro32->setNrInscricao($this->_nrInscricao);
                                $this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
                                $this->_registro32->setTpInscricao($this->_nrTpInscricao);
                                $this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                                $this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
                                $this->_registro32->setCodMovimentacao($codFGTS);
                                $this->_registro32->setDtMovimentacao($dtinicial);
                                $this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
                                
                                $this->_gfipMagnetico->insereRegistroLista(
                                    $this->_nrOrg, $this->_mesCompetencia, Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao,
                                    $this->_nrInscricao, $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
                                    $this->_nrInscricaoTomador, $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(),
                                    $nrPisPasepCi, $vrSalarioFamilia, $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador
                                );
                                $this->_registro32->setLinhaRegistro();
                                $gerouMovimento = true;
                                
                                if ($codFGTS == self::CODFGTS_SITUFUNC_PRORROGACAOLICENCAMATERNIDADE) {
                                    
                                    $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findOneBy(array('nrorg' => $this->_nrOrg, 'nrvinculom' => $vinculo->getNrvinculom(), 'nrsitufuncm' => self::NRSITUFUNC_LICENCAMATERNIDADE));
                                    
									if (is_object($altesitufunc)) {

										$dtfinal = $altesitufunc->getDtfimsitufunc();
										$dtfinal = DateUtil::getDataDeString($dtfinal);

										if ($dtfinal->format('d/m/Y') === $dtinicial->format('d/m/Y')) {
											$dtinicial = $altesitufunc->getDtinisitufunc();
											$this->_registro32->setNrInscricao($this->_nrInscricao);
											$this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
											$this->_registro32->setTpInscricao($this->_nrTpInscricao);
											$this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
											$this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
											$this->_registro32->setCodMovimentacao(self::CODFGTS_SITUFUNC_LICENCAMATERNIDADE);
											$this->_registro32->setDtMovimentacao($dtinicial);
											$this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
											$this->_gfipMagnetico->insereRegistroLista(
											$this->_nrOrg, $this->_mesCompetencia, Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao,
											$this->_nrInscricao, $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
											$this->_nrInscricaoTomador, $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(),
											$nrPisPasepCi, $vrSalarioFamilia, $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador);
                                            
											$this->_registro32->setLinhaRegistro();
										}
									}
								}

                            } else {
                                MsgCalculo::getInstance()->adicionaMensagem('Codigo de FGTS nao parametrizado na situacao funcional para o vinculo '.$vinculo->getNrvinculom());
                            }
                        }
                    }
                }

                if (empty($dtFim) === false) {
                    if (($gerouMovimento === true)
                        && (DateUtil::isBetween($dtFim, $this->_dtInicio, FPA::getUltimoDiaMes())))
                    {
                        $codFGTS = $situacaoFuncional->historico->getCodfgtsret();
                        
                        if (empty($codFGTS) !== true) {
                            $this->_registro32->setNrInscricao($this->_nrInscricao);
                            $this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
                            $this->_registro32->setTpInscricao($this->_nrTpInscricao);
                            $this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                            $this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
                            $this->_registro32->setCodMovimentacao($codFGTS);
                            $this->_registro32->setDtMovimentacao($dtFim);
                            $this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
                            
                            $this->_gfipMagnetico->insereRegistroLista(
                                $this->_nrOrg, $this->_mesCompetencia, Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao,
                                $this->_nrInscricao, $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador,
                                $this->_nrInscricaoTomador, $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(),
                                $nrPisPasepCi, $vrSalarioFamilia, $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador
                            );
                            $this->_registro32->setLinhaRegistro();
                        }
                    }
                }
            }
        }
        
        if ($item->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_NORMAL
            || $item->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_DEMITIDO) {
           
            $situfuncm = null;//$this->_sitFuncional->retornaSituacaoFuncional($this->_nrOrg, $this->_nrOrg, $item->getNrsitufuncm());
            
            if (is_object($situfuncm)) {
                
                $codFGTS = $situfuncm->historico->getCodfgts();
                
                if(empty($codFGTS) !== true) {
                    $this->_registro32->setNrInscricao($this->_nrInscricao);
                    $this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
                    $this->_registro32->setTpInscricao($this->_nrTpInscricao);
                    $this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                    $this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
                    $this->_registro32->setCodMovimentacao($codFGTS);
                    $this->_registro32->setDtMovimentacao($item->getDtinisitufunc());
                    $this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
                    
                    $this->_gfipMagnetico->insereRegistroLista($this->_nrOrg, $this->_mesCompetencia,
                    Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao, $this->_nrInscricao,
                    $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador, $this->_nrInscricaoTomador,
                    $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $nrPisPasepCi, $vrSalarioFamilia,
                    $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador);
                    
                    $this->_registro32->setLinhaRegistro();
               }
           }
       }
       
       $this->_codMovimentacao = $codFGTS;

        $tranferenciaDemissao = array('8','9');

        $inicioMovimentacao = DateUtil::getDataDeString($gpemovimentacao->getDtinimoviment());

        if ((array_search($gpemovimentacao->getNrtpmovtransfm(), $tranferenciaDemissao) !== false) &&
            (($inicioMovimentacao > $this->_dtInicio) ||
                ($inicioMovimentacao->format('d/m/Y') == $this->_dtInicio->format('d/m/Y'))) &&
            (($gpemovimentacao->getNrtpmovtransfm() == self::TRANSFERENCIA_ENTRADA) ||
                ($gpemovimentacao->getNrtpmovtransfm() == self::TRANSFERENCIA_SAIDA))
        ) {
            $inicioAdmissao = DateUtil::getDataDeString($vinculo->getDtadmissaovinc());
            if ($inicioAdmissao < $inicioMovimentacao) {
                $codMovimentacao = $this->entityManager->getRepository(Repositories::GPE_TPMOVTRANSFH)->findOneBy(array('nrtpmovtransfm' => $gpemovimentacao->getNrtpmovtransfm(),'nrtpmovtransfh' => $gpemovimentacao->getNrtpmovtransfm()));

                $this->_registro32->setNrInscricao($this->_nrInscricao);
                $this->_registro32->setNrInscricaoTomador($this->_nrInscricaoTomador);
                $this->_registro32->setTpInscricao($this->_nrTpInscricao);
                $this->_registro32->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
                $this->_registro32->setNrCategoria($this->_nrCategoriaTrabalhador);
                $this->_registro32->setCodMovimentacao($codMovimentacao->getCodfgts());
                $this->_registro32->setDtMovimentacao($gpemovimentacao->getDtinimoviment());
                $this->_registro32->geraRegistro32($pessoa, $vinculo, $this->entityManager, $this->hcmParameters);
                
                $this->_gfipMagnetico->insereRegistroLista($this->_nrOrg, $this->_mesCompetencia,
                    Registro32SEFIP::REGISTRO_32, $this->_nrTpInscricao, $this->_nrInscricao,
                    $this->_registro32->getLinhaRegistro(), $this->_cdOperador, $this->_nrTpInscricaoTomador, $this->_nrInscricaoTomador,
                    $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $nrPisPasepCi, $vrSalarioFamilia,
                    $vrSalarioMaternidade, $this->_nrCategoriaTrabalhador);
                $this->_registro32->setLinhaRegistro();
            }
        }
    }

    private function _calculaBasesEstrutSEFIP($nrEstrutura)
    {
        $this->_dadoEstrutura = array();
        foreach ($this->_listaCalculoFolha as $nrCalculo) {
            $contador = $this->_itemCalcEstrut->retornaItensPorEstrutura($this->_nrOrg, $nrEstrutura, $nrCalculo);
            $this->_calculaCompensacao();
            $this->_calculaValorCooperativas();
            $this->_calculaValorRetencao();
        }
    }

    private function _calculaBasesSEFIP($nrVinculo, $dtRescisao=null)
    {
        $this->_dadoVinculo = array();
        foreach ($this->_listaCalculoFolha as $nrCalculo) {
            
            $calculo = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrorg' => $this->_nrOrg, 'nrcalculofolha' => $nrCalculo));
            $contador = $this->_itemCalculo->retornaItensPorVinculo($this->_nrOrg, $nrVinculo, $nrCalculo);
            $this->_calculaSalarioFamilia();
            $this->_calculaSalarioMaternidade();
            $this->_calculaRemuneracaoSem13Salario();
            $this->_calculaRemuneracao13SalarioMensal();
            $this->_calculaRemuneracaoContribuicaoPrevidenciaria();
            $this->_calculaRemuneracao13SalarioCompetenciaMovimento();
            if ($calculo->getNrocorrecal() > 1)
                $this->_calculaBase13SalarioComplemento();
            $this->_calculaRemuneracao13SalarioRescisaoCompetenciaMovimento();
            if ($calculo->getNrocorrecal() == 1)
                $this->_calculaRemuneracao13SalarioGPSCompetencia13();
            $this->_calculaInssMultiplosVinculos();
            if (!is_null($dtRescisao) || $this->_competencia13) {
            $this->_calculaSalarioMaternidade13();
            }
            $this->_calculaContribuicaoInssDescontada();
            $this->_calculaDeducaoInssPatronal();
            $this->_calculaValorDescontadoSegurado();
            $this->_calculaContribuicaoPrevidenciaria();
        }
    }

    private function _calculaValorItemGrupoEstrut($referencia)
    {
        $valor = $this->_itemGrupo->calculaValorItemGrupoIncidencia(
            $this->_mesCompetencia, $this->_itemCalcEstrut, $this->_evento, $referencia
        );
        return $valor;
    }

    private function _calculaValorItemGrupo($referencia)
    {
        $valor = $this->_itemGrupo->calculaValorItemGrupoIncidencia(
            $this->_mesCompetencia, $this->_itemCalculo, $this->_evento, $referencia
        );
        return $valor;
    }

    private function _calculaSalarioFamilia()
    {
        if ($this->_competencia13)
            $valor = 0;
        else
            $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_SALARIO_FAMILIA);

        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_SALARIO_FAMILIA, $valor);
    }

    private function _calculaSalarioMaternidade()
    {
        if ($this->_competencia13)
            $valor = 0;
        else
            $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE);

        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE, $valor);
    }

    private function _calculaRemuneracaoSem13Salario()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_REM_SEM_13SAL);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_REM_SEM_13SAL, $valor);
    }

    private function _calculaRemuneracao13SalarioMensal()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_13SAL_PAGO_MES);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_13SAL_PAGO_MES, $valor);
    }

    private function _calculaRemuneracaoContribuicaoPrevidenciaria()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_REM_BASE_GPS);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_REM_BASE_GPS, $valor);
    }

    private function _calculaRemuneracao13SalarioCompetenciaMovimento()
    {
        if ($this->_competencia13)
            $valor = 0;
        else
            $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_REM_13SAL_MOV);

        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_REM_13SAL_MOV, $valor);
    }

    private function _calculaBase13SalarioComplemento()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_COMPLEM_INSS_13SAL);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_COMPLEM_INSS_13SAL, $valor);
    }

    private function _calculaRemuneracao13SalarioGPSCompetencia13()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_REM_13SAL_GPS);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_REM_13SAL_GPS, $valor);
    }

    private function _calculaInssMultiplosVinculos()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_INSS_MULT_VINCULOS);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_INSS_MULT_VINCULOS, $valor);
    }

    private function _calculaCompensacao()
    {
        $valor = $this->_calculaValorItemGrupoEstrut(RegistroSEFIP::SEFIP_COMPEN_VLRCORR);
        $this->_consolidaDadosEstrut(RegistroSEFIP::SEFIP_COMPEN_VLRCORR, $valor, $this->_itemCalcEstrut->getMenorDtOcorrencia(), $this->_itemCalcEstrut->getMaiorDtOcorrencia());
    }

    private function _calculaValorCooperativas()
    {
        $valor = $this->_calculaValorItemGrupoEstrut(RegistroSEFIP::SEFIP_VALOR_NF_COOPER);
        $this->_consolidaDadosEstrut(RegistroSEFIP::SEFIP_VALOR_NF_COOPER, $valor);
    }

    private function _calculaValorRetencao()
    {
        $valor = $this->_calculaValorItemGrupoEstrut(RegistroSEFIP::SEFIP_VALOR_RETENCAO);
        $this->_consolidaDadosEstrut(RegistroSEFIP::SEFIP_VALOR_RETENCAO, $valor);
    }

    private function _calculaSalarioMaternidade13()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE_13);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE_13, $valor);
    }

    private function _calculaContribuicaoInssDescontada()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_CONTRIB_INSS_DESC);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_CONTRIB_INSS_DESC, $valor);
    }

    private function _calculaDeducaoInssPatronal()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_DEDUCAO_INSS_PATRONAL);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_DEDUCAO_INSS_PATRONAL, $valor);
    }

    private function _calculaValorDescontadoSegurado()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_VALOR_DESC_SEGURADO);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_VALOR_DESC_SEGURADO, $valor);
    }

    private function _calculaContribuicaoPrevidenciaria()
    {
        $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_CONTRIB_PREVIDENCIA);
        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_CONTRIB_PREVIDENCIA, $valor);
    }

    private function _calculaRemuneracao13SalarioRescisaoCompetenciaMovimento()
    {
        if ($this->_competencia13)
            $valor = $this->_calculaValorItemGrupo(RegistroSEFIP::SEFIP_REM_13SAL_MOV_13);
        else
            $valor = 0;

        $this->_consolidaDadosVinculo(RegistroSEFIP::SEFIP_REM_13SAL_MOV_13, $valor);
    }

    private function _consolidaDadosEstrut($referencia, $valor, $dtinicial = null, $dtfinal = null)
    {
        $this->_dadoEstrutura[$referencia] += $valor;
        
        if (!is_null($dtinicial)) $this->_dadoEstrutura['DTINICIAL'] = $dtinicial->format('Y/m');
        if (!is_null($dtfinal)) $this->_dadoEstrutura['DTFINAL'] = $dtfinal->format('Y/m');
    }

    private function _consolidaDadosVinculo($referencia, $valor)
    {
        $this->_dadoVinculo[$referencia] += $valor;
    }

    private function _consolidaDadosSEFIP($tpRegistro, $nrInscricao, $nrInscricaoTomador)
    {
        $posicao = $tpRegistro . '.' . $nrInscricao . '.' . $nrInscricaoTomador;
        if ($tpRegistro === Registro10SEFIP::REGISTRO_10) {
            $this->_consolidaDados(
                $posicao, RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE, $this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE]
            );
        }
        elseif ($tpRegistro === Registro12SEFIP::REGISTRO_12) {
            $this->_consolidaDados(
                $posicao, RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE_13, $this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_MATERNIDADE_13]
            );
        }
        $this->_consolidaDados(
            $posicao, RegistroSEFIP::SEFIP_SALARIO_FAMILIA, $this->_dadoVinculo[RegistroSEFIP::SEFIP_SALARIO_FAMILIA]
        );
    }

    private function _consolidaDados($posicao, $referencia, $valor)
    {
        $this->_dadoAlterado[$posicao][$referencia] += $valor;
    }

    public function modificaValoresConsolidados()
    {

        foreach ($this->_dadoAlterado as $posicao => $valor) {

            if ($posicao !== 0) {
                list($tpRegistro, $nrInscricao, $nrInscricaoTom) = explode('.', $posicao);
                $posRegistro = $this->_gfipMagnetico->procuraRegistroLista($nrInscricao, $nrInscricaoTom, $tpRegistro, $this->_nrOrg, $this->_cdOperador);
                $registroMag = $this->_gfipMagnetico->getItem($posRegistro);
                if ($tpRegistro === Registro10SEFIP::REGISTRO_10) {
                    $this->_registro10->alteraDadoRegistro10($registroMag, $valor);
                } elseif ($tpRegistro === Registro12SEFIP::REGISTRO_12) {
                    $this->_registro12->alteraDadoRegistro12($registroMag, $valor);
                } elseif ($tpRegistro === Registro20SEFIP::REGISTRO_20) {
                    $this->_registro20->alteraDadoRegistro20($registroMag, $valor);
                }
            }
        }
    }

    public function retroceder()
    {
        $this->_gfipMagnetico->retroceder($this->_nrOrg, $this->_mesCompetencia, $this->_cdOperador);
    }
}