<?php

namespace HCM\FPA\Registro\sefip;

use HCM\FPA\Registro\RegistroSEFIP;
use HCM\Util\DateUtil;
use HCM\FPA\Entity\FpagrrfmagnetEntity;

class Registro32SEFIP extends RegistroSEFIP
{

    const REGISTRO_32 = '32';
    private $_fgtsmesanterior;
    private $_pessoa;
    private $_vinculo;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtFimPosterior;
    private $_mesAnterior;
    private $_mesPosterior;
    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_nrCategoria;
    private $_competencia13;
    private $_codMovimentacao;
    private $_dtMovimentacao;

    public function geraRegistro32($pessoa, $vinculo, $entityManager, $hcmParameters)
    {

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_fgtsmesanterior = new FpagrrfmagnetEntity($entityManager, $hcmParameters);

        $this->_vinculo = $vinculo;
        $this->_pessoa = $pessoa;

        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia());
        $this->_dtInicio = $this->_mesCompetencia;
        $this->_dtFim = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);

        $this->_mesAnterior = DateUtil::subtraiIntervalo($this->_mesCompetencia, 1, DateUtil::MES);
        $this->_mesPosterior = DateUtil::adicionaIntervalo($this->_mesCompetencia, 1, DateUtil::MES);
        
        $this->_dtFimPosterior = DateUtil::getUltimoDiaDoMes(DateUtil::adicionaIntervalo($this->_mesCompetencia, 1, DateUtil::MES));
        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_competencia13 = $this->getCompetencia13();

        $this->_geraRegistro();
    }

    private function _geraRegistro()
    {
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_32, $this->_tpInscricao, $this->_nrInscricao);

        // Bloco 4 | 018 | Tipo de Inscricao do tomador.
        $this->atribuiDado($this->_tpInscricaoTomador, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 5 | 019-032 | Inscricao do tomador.
        $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 6 | 033-043 | PIS/PASEP/CI.
        $identificador = $this->_pessoa->historico->getNrpispaseppes();
        $this->atribuiDado($identificador, 11);

        // Bloco 7 | 044-051 | Data de Admissao.
        // $dataAdmissao = new TkDateTime($this->_vinculo->getDtadmissaovinc());.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        // Bloco 8 | 052-053 | Categoria do Trabalahador.
        $this->atribuiDado($this->_nrCategoria, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 9 | 054-123 | Nome do trabalhador.
        $nome = $this->_pessoa->historico->getNmpessoa();
        $this->atribuiDado($nome, 70, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 124-125 | Codigo de movimentacao.
        // $codigoMovimentacao = 'N1'; // Transfer�ncia do empregado para outro estabelecimento da mesma empresa.
        $this->_atribuiCodigoMovimentacao();

        // Bloco 11 | 126-133 | Data da movimentacao.
        $this->_atribuiDtMovimentacao();
        // Bloco 12 | 134 | Indicativo de recolhimento de FGTS.
        $recolhimentoFGTS = parent::CARACTER_BRANCO;
        $this->_atribuiRecolhimentoFGTS();

        // Bloco 13 | 135-359 | Brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 225, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 14 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);
    }

    public function setCodMovimentacao($codMovimentacao)
    {
        $this->_codMovimentacao = $codMovimentacao;
    }

    public function setDtMovimentacao($dtMovimentacao)
    {
        $this->_dtMovimentacao = $dtMovimentacao;
    }

    public function setNrCategoria($nrCategoria)
    {
        $this->_nrCategoria = $nrCategoria;
    }

    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }

    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }

    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }

    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }

    private function _atribuiDataAdmissao($dtAdmissao)
    {
        // Obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataAdmissao = DateUtil::getDataDeString($dtAdmissao);
            $dataAdmissao = $dataAdmissao->format('d/m/Y');
        } else {
            $dataAdmissao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataAdmissao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataAdmissao);
    }

    private function _atribuiCodigoMovimentacao()
    {
        $codCategorias = array(
            '01', '02', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $this->_codMovimentacao = self::CARACTER_BRANCO;
        }

        $this->atribuiDado($this->_codMovimentacao, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    private function _atribuiDtMovimentacao()
    {
        if (!empty($this->_dtMovimentacao)) {

            $dataMovimentacao = DateUtil::getDataDeString($this->_dtMovimentacao);
            $dataAdmissao =  DateUtil::getDataDeString($this->_vinculo->getDtadmissaovinc());

            // Data movimentação deve ser maior que a data de admissão
            if ($dataMovimentacao > $dataAdmissao
                || $dataMovimentacao->format('d/m/Y') === $dataAdmissao->format('d/m/Y')
            ) {
                // Deve estar compreendida no mêsimediatamente anterior ou no mês da competência, para os códigos de movimentação H,J, K, M, N1, N2, S2, S3 e U1.
                $codMovimentacao = Array('H', 'J', 'K', 'M', 'N1', 'N2', 'S2', 'S3', 'U1');
                if (array_search($this->_codMovimentacao, $codMovimentacao) !== false) {
                    if ($dataMovimentacao > $this->_mesAnterior
                        || $dataMovimentacao < $this->_dtFim
                        || $dataMovimentacao->format('d/m/Y') === $this->_dtFim->format('d/m/Y')
                    ) {
                        $dataMovimentacao = $dataMovimentacao;
                    }
                }
                // Deve estar compreendida no mês da competência,se o código de movimentação for Z1, Z2, Z3, Z4, Z5 e Z6.
                $codMovimentacao = Array('Z1', 'Z2', 'Z3', 'Z4', 'Z5', 'Z6');
                if (array_search($this->_codMovimentacao, $codMovimentacao)) {
                    if ($dataMovimentacao > $this->_dtInicio
                        || $dataMovimentacao < $this->_dtFim
                    ) {
                        $dataMovimentacao = $dataMovimentacao;
                    }
                }
                // Deve estar compreendida no mês anterior, no mês da competência ou no mês posterior (se o recolhimento do FGTS já tiversido efetuado) e o código de movimentação for I1, I2, I3 , I4, I5 ou L.
                $codMovimentacao = Array('I1', 'I2', 'I3', 'I4', 'I5', 'L');
                if (array_search($this->_codMovimentacao, $codMovimentacao)) {
                    if ($dataMovimentacao > $this->_mesAnterior
                        || $dataMovimentacao < $this->_dtFimPosterior
                    ) {
                        $dataMovimentacao = $dataMovimentacao;
                    }
                }
                // Deve ser menor ou igual ao mês de competência , para códigos de movimentação O1, O2, O3, P1, P2, Q1, Q2, Q3, Q4, Q5, Q6, R, U3. W, X e Y.
                $codMovimentacao = Array('O1', 'O2', 'O3', 'P1', 'P2', 'Q1', 'Q2', 'Q3', 'Q4', 'Q5', 'Q6', 'R', 'U3', 'W', 'X', 'Y');
                
                if (array_search($this->_codMovimentacao, $codMovimentacao)) {
                    if ($dataMovimentacao < $this->_mesCompetencia
                        || $dataMovimentacao->format('d/m/Y') === $this->_mesCompetencia->format('d/m/Y')
                    ) {
                        $dataMovimentacao = $dataMovimentacao;
                    }
                }
                // Deve ser informada para os códigos de movimentação O1, O2, O3, Q1, Q2, Q3, Q4, Q5, Q6 e R, mensalmente, até que se dê o efetivo retorno.
                $codMovimentacao = Array('O1', 'O2', 'O3', 'Q1', 'Q2', 'Q3', 'Q4', 'Q5', 'Q6', 'R');
                if (array_search($this->_codMovimentacao, $codMovimentacao)) {
                    //$dataMovimentacao = $this->_mesCompetencia;
                    $dataMovimentacao = $dataMovimentacao;
                }
//                sempre que informado o código de movimentação V3 a data de movimentação a ser informada é a do afastamento definitivo.
                if ($this->_codMovimentacao === "V3") {
                    if ($this->_vinculo->getDtrescisaovinc()) {
                        $dataMovimentacao = DateUtil::getDataDeString($this->_vinculo->getDtrescisaovinc());
                    } else {
                        $dataMovimentacao = $dataMovimentacao;
                    }
                }
//                verifica todos - @todo provisório - ver melhor forma para isto.
                $array = Array('H', 'J', 'K', 'M', 'N1', 'N2', 'N3', 'S2', 'S3', 'U1', 'Z1', 'Z2', 'Z3', 'Z4', 'Z5', 'Z6', 'I1',
                    'I2', 'I3', 'I4', 'I5', 'L', 'P1', 'P2', 'P3', 'Q1', 'Q2', 'Q3', 'Q4', 'Q5', 'Q6', 'R', 'U3',
                    'W', 'X', 'Y', 'O1', 'O2', 'O3', 'Q1', 'Q2', 'Q3', 'Q4', 'Q5', 'Q6', 'R', 'V3');

                if (array_search($this->_codMovimentacao, $array) === false) {
                    $dataMovimentacao = $this->_mesCompetencia;
                }
                $this->atribuiDado($dataMovimentacao->format('d/m/Y'), 8,
                    parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            } else {
                $this->atribuiDado($dataMovimentacao->format('d/m/Y'), 8,
                    parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            }
        }
    }

    private function _atribuiRecolhimentoFGTS()
    {

        //a) Caracteres possíveis: “S” ou “s”, “N” ou “n”, “C” ou “c” e “Branco”.
        //b) Só deve ser informado “S’ ou “N” para competência maior que 01/1998.
        //c) Obrigatório “S’ ou “N” para códigos de movimentação I1, I2, I3, I4, I5 ou L. Não pode ser informado “S’ ou “N” para os demais códigos de movimentação.
        //d) Deve ser informado “S” apenas para as categorias 01, 03, 04, 05, 06 e 07.
        //e) Só deve ser informado “C’ para competência maior ou igual a 10/1998.
        //f) Não deve ser informado “C” para os códigos 640 e 660.
        //g) Deve ser informado “C” apenas para as categorias 01, 02, 03, 04, 05, 06 e 07.
        //h) Não pode ser informado “S”, “N” ou “C para a competência 13.
        //i) Sempre que não informado campo deve ficar em branco.

        $categoriasc = array('01', '02', '03', '04', '05', '06', '07');
        $categoriass = array('01', '03', '04', '05', '06', '07');
        $codrecolhimento = array('640', '660');
        $codmovimentacao = array('I1', 'I2', 'I3', 'I4', 'I5', 'L');

        if ($this->_competencia13) {
            $recolhimentoFGTS = self::CARACTER_BRANCO;
        } else {
            //verifica codigo de movimentacao, se existir retorna "S" - ( veja letra "c" );
            if (array_search($this->_codMovimentacao, $codmovimentacao) !== false) {
                //verifica codigo da categoria, se NÂO existir retorna "C" - ( veja letra "f" );
                if (array_search($this->_nrCategoria, $categoriass) !== false) {
                    $recolhimentoFGTS = self::SIM;
                } else {
                    $recolhimentoFGTS = self::NAO;
                }
            } else {
                //verifica codigo de recolhimento, se NÂO existir retorna "C" - ( veja letra "f" );
                if ((!array_search($this->_nrRecolhimento, $codrecolhimento))
                    //verifica codigo da categoria do trabalhador, se existir retorna "C" - ( veja letra "g" );
                    && (array_search($this->_nrCategoria, $categoriasc) !== false)
                ) {
//                    @todo - verificar novamente.
//                    $recolhimentoFGTS = self::CARACTER_C;
                    $recolhimentoFGTS = self::CARACTER_BRANCO;
                } else {
                    //senao retorna branco
                    $recolhimentoFGTS = self::CARACTER_BRANCO;
                }
            }
            $this->atribuiDado($recolhimentoFGTS, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        }
    }
}