<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Repository\ResumoContabilRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Geral\Controllers\RelatoriosHcmController;

class ResumoContabilService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ResumoContabilRepository $resumoContabilRepository, RelatoriosHcmController $relatoriosHcmController) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->resumoContabilRepository = $resumoContabilRepository;
        $this->relatoriosHcmController = $relatoriosHcmController;
    }

    public function geraArquivoLotePadrao($dtocorrencia, $nrorg, $cdoperador, $response, $dtcontabil, $nrtipomoviment = null, $listaestrutsup = null, $nrtipohieraqorg = null)
    {
        try {
            $estruturas = $this->resumoContabilRepository->getEstruturasSupResCont($dtocorrencia, $nrorg, $nrtipomoviment, $listaestrutsup);
            foreach ($estruturas as &$estrutura) {

                //$nrestrutsuperior = $hierarquia_org->determinaEstruturaSuperior($estrutura['NRESTRUTURAM'], $hierarquias);
    
                if (is_null($estrutura['NRESTRUTURAMSUP'])) {
                    $message = "Existem resumos sem a estrutura superior. Verifique a hierarquia e realize a contabilização novamente.";
                    $nrEstrutura = null;
                    $nmEstrutura = 'SEM_ESTRUTURA';
    
                    $nome_arquivo = 'integracao_resumo_contabil_'.$nmEstrutura;
                }
                else {
                    $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
                        array(
                            'nrestruturam' => $estrutura['NRESTRUTURAMSUP'],
                            'nrorg' => $nrorg
                        )
                    );
                    $nrEstrutura = $estruturam->getNrestruturam();
                    $nmEstrutura = $estruturam->getNmestruturam();
    
                    $nome_arquivo = 'integracao_resumo_contabil_'.$nrEstrutura.'-'.$nmEstrutura;
                }
    
                $nome_arquivo = str_replace( array(' ','\\','/',':','*','?','"','<','>','|','&'), '', $nome_arquivo);
    
                $filtro = array(
                    'P_ESTRUTURA' => $nrEstrutura,
                    'P_NRORG' => $nrorg,
                    'P_OCORRENCIA' => $dtocorrencia,
                    'P_NRTIPOMOVIMENT' => $nrtipomoviment,
                    'P_COMPETENCIA'=>$this->hcmParameters->getDtCompetencia(),
                    'download' => array(
                              'fileName' => $nome_arquivo,
                              'format' => "txt",
                              'strFind' => "·",
                              'strReplace' => "\r"
                            )
                    );
                    
                if($dtcontabil){
                    $filtro['P_DTCONTABIL'] = $dtcontabil;
                }

                $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'integracao_resumo_contabil', null, $response);
                
            }
        
        } catch (\Exception $e) {
            throw $e;
        }
    }

} 
