<?php
namespace HCM\Geral\Services;

use Teknisa\Model\Gestaopessoas\Entities\Base\GpeSitufuncm;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\AlteSituacaoFuncionalService;
use HCM\Geral\Repository\SolicitacaoRepository;
use HCM\Geral\Services\VinculoServices;
use HCM\Geral\Services\PessoaService;
use HCM\Geral\Services\MovimentacaoService;
use HCM\BTL\Services\ProcessosService;
use HCM\FPA\Services\DependenteVincService;

use HCM\FPA\Repository\GozoFeriasRepository;
use HCM\Geral\Repository\PessoaRepository;

use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Messages;
use HCM\Util\Util;

use Zeedhi\Framework\DTO\Response;



class SolicitacaoService {

    const ALTERA_VINCULO = 'ALTERA';

    const TP_SOLI_FERIAS = 1;
    const TP_SOLI_MOVIM = 2;
    const TP_SOLI_SALARIO = 3;
    const TP_SOLI_TREINAMENTO = 4;
    const TP_SOLI_DESLIGAMENTO = 5;
    const TP_SOLI_PESSOAL = 6;
    const TP_SOLI_ALTERADADOS = 7;
    const TP_SOLI_ALTERAENDERECO = 8;
    const TP_SOLI_OCUPACAO = 9;
    const TP_SOLI_VTRANS = 10;
    const TP_SOLI_DEP = 11;
    const TP_SOLI_ALTESITU = 12;
    const TP_SOLI_PLANOSAUDE = 13;
    const TP_SOLI_ALTERAESCALA = 14;
    const TP_SOLI_ALTERACONTA = 15;
    const TP_SOLI_ALTERACOMU = 16;
    const TP_SOLI_ADMISSAO = 17;


    private $_fieldsFerias;
    private $_fieldsMovimentacao;
    private $_fieldsSalario;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, SolicitacaoRepository $solicitacaoRepository,
                                GozoFeriasRepository $gozoFeriasRepository,VinculoServices $vinculoServices, DependenteVincService $dependenteVincService,
                                AlteSituacaoFuncionalService $alteSituacaoFuncionalService, PessoaService $pessoaService, PessoaRepository $pessoaRepository,
                                ProcessosService $processosService, MovimentacaoService $movimentacaoService
                                ) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->solicitacaoRepository = $solicitacaoRepository;
        $this->pessoaRepository = $pessoaRepository;
        $this->gozoFeriasRepository = $gozoFeriasRepository;
        $this->vinculoServices = $vinculoServices;
        $this->dependenteVincService = $dependenteVincService;
        $this->alteSituacaoFuncionalService = $alteSituacaoFuncionalService;
        $this->pessoaService = $pessoaService;
        $this->pessoaRepository = $pessoaRepository;
        $this->processosService = $processosService;
        $this->movimentacaoService = $movimentacaoService;



        $this->fpaGozoferias = Repositories::FPA_GOZOFERIAS;
        $this->gpeAltesalrio = Repositories::GPE_ALTESALARIO;
        $this->gpeAlteocupacao = Repositories::GPE_ALTEOCUPACAO;
        $this->gpeTreinamento = Repositories::GPE_TREINAMENTO;
        $this->vabLinhavinc = Repositories::VAB_LINHAVINC;
        $this->gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
        $this->alteSituFunc = Repositories::GPE_ALTESITUFUNC;
        $this->beneficiario = Repositories::VAB_BENEFICIARIO;
        $this->pessoah = Repositories::GPE_PESSOAH;
        $this->enderecoParc = Repositories::ENDERECOPARC;
        $this->comunicaParc = Repositories::COMUNICAPARC;
        $this->contaCorrenteParc = Repositories::CONTCORRPARC;
        $this->alteEscala = Repositories::GPE_ALTEESCALA;



        $this->_fieldsFerias = array(
            array('DSCHAVEITEMSOLI' => 'DTINIGOZOFERIAS', 'DSLABELITEMSOLI' => 'Início Gozo'),
            array('DSCHAVEITEMSOLI' => 'QTDEDIASFERIAS', 'DSLABELITEMSOLI' => 'Qtde de Dias'),
            array('DSCHAVEITEMSOLI' => 'DTFIMGOZOFERIAS', 'DSLABELITEMSOLI' => 'Fim do Gozo'),
            array('DSCHAVEITEMSOLI' => 'DTRETORNOFERIAS', 'DSLABELITEMSOLI' => 'Data de Retorno'),
            array('DSCHAVEITEMSOLI' => 'IDADIANT13SALAR', 'DSLABELITEMSOLI' => 'Adiantar 13º?'),
            array('DSCHAVEITEMSOLI' => 'ABONOPECUNIARIO', 'DSLABELITEMSOLI' => 'Abono Pecuniário'),
        );

        $this->_fieldsMovimentacao = array(
            array('DSCHAVEITEMSOLI' => 'NRTIPOTRANSFER', 'DSLABELITEMSOLI' => 'Tipo de Transferência'),
            array('DSCHAVEITEMSOLI' => 'NRTPMOVTRANSFM', 'DSLABELITEMSOLI' => 'Transferência'),
            array('DSCHAVEITEMSOLI' => 'NRMOTIVOTRANSF', 'DSLABELITEMSOLI' => 'Motivo Transferência'),
            array('DSCHAVEITEMSOLI' => 'NRESTRUTURAM', 'DSLABELITEMSOLI' => 'Estrutura'),
            array('DSCHAVEITEMSOLI' => 'DTINIMOVIMENT', 'DSLABELITEMSOLI' => 'Data de Início'),
            array('DSCHAVEITEMSOLI' => 'DTFIMMOVIMENT', 'DSLABELITEMSOLI' => 'Data Final'),
        );

        $this->_fieldsSalario = array(
            array('DSCHAVEITEMSOLI' => 'DTALTESALARIO', 'DSLABELITEMSOLI' => 'Data da Alteração'),
            array('DSCHAVEITEMSOLI' => 'NRTIPOALTERA', 'DSLABELITEMSOLI' => 'Tipo de Alteração'),
            array('DSCHAVEITEMSOLI' => 'NRTPMODALIDSAL', 'DSLABELITEMSOLI' => 'Modalidade'),
            array('DSCHAVEITEMSOLI' => 'IDTPSALARIO', 'DSLABELITEMSOLI' => 'Tipo de Salário'),
            array('DSCHAVEITEMSOLI' => 'NROCUPACAOM_SAL', 'DSLABELITEMSOLI' => 'Ocupação'),
            array('DSCHAVEITEMSOLI' => 'NOVOVALORSALARIO', 'DSLABELITEMSOLI' => 'Novo Valor Salarial')
        );
        
        $this->_fieldsOcupacao = array(
            array('DSCHAVEITEMSOLI' => 'NROCUPACAOM_OCU', 'DSLABELITEMSOLI' => 'Ocupação'),
            array('DSCHAVEITEMSOLI' => 'NRMOTIVOOCUPA', 'DSLABELITEMSOLI' => 'Motivo'),
            array('DSCHAVEITEMSOLI' => 'NRTPMODALIDSAL', 'DSLABELITEMSOLI' => 'Modalidade'),
            array('DSCHAVEITEMSOLI' => 'NRTIPOOCUPACAO', 'DSLABELITEMSOLI' => 'Tipo de Ocupação'),
            array('DSCHAVEITEMSOLI' => 'DTINIOCUPACAO', 'DSLABELITEMSOLI' => 'Início da Ocupação'),
            
        );
        
        $this->_fieldsTreinamento = array(
            array('DSCHAVEITEMSOLI' => 'NRCURSO', 'DSLABELITEMSOLI' => 'Curso'),
            array('DSCHAVEITEMSOLI' => 'NRPERFACESAPROV', 'DSLABELITEMSOLI' => 'Perfil de Aprovação'),
            array('DSCHAVEITEMSOLI' => 'NRMETODOTRE', 'DSLABELITEMSOLI' => 'Metodo'),
            array('DSCHAVEITEMSOLI' => 'NMTREINAMENTO', 'DSLABELITEMSOLI' => 'Nome do Treinamento'),
            array('DSCHAVEITEMSOLI' => 'NRPERFACESEXEC', 'DSLABELITEMSOLI' => 'Perfil de Execução'),
            array('DSCHAVEITEMSOLI' => 'IDTREINAMENTOINT', 'DSLABELITEMSOLI' => 'Treinamento Interno'),
            array('DSCHAVEITEMSOLI' => 'DTINIPLANEJADOTRE', 'DSLABELITEMSOLI' => 'Inicio Planejado'),
            array('DSCHAVEITEMSOLI' => 'DTFIMPLANEJADOTRE', 'DSLABELITEMSOLI' => 'Fim Planejado'),
        );
        
        $this->_fieldsValeTransporte = array(
            array('DSCHAVEITEMSOLI' => 'NRLINHATRANSP', 'DSLABELITEMSOLI' => 'Linha'),
            array('DSCHAVEITEMSOLI' => 'NRTIPOLINHAVINC', 'DSLABELITEMSOLI' => 'Tipo da Linha'),
            array('DSCHAVEITEMSOLI' => 'NRSENTIDOLINHA', 'DSLABELITEMSOLI' => 'Sentido da Linha'),
            array('DSCHAVEITEMSOLI' => 'QTLINHATARIFA', 'DSLABELITEMSOLI' => 'Quantidade'),
            array('DSCHAVEITEMSOLI' => 'DTINILINHAVINC', 'DSLABELITEMSOLI' => 'Inicio de Validade'),
            array('DSCHAVEITEMSOLI' => 'DTFIMLINHAVINC', 'DSLABELITEMSOLI' => 'Fim de Validade'),
        );
        $this->_fieldsDependente = array(
            array('DSCHAVEITEMSOLI' => 'NRTIPORELACIONA', 'DSLABELITEMSOLI' => 'Tipo de relacionamento'),
            array('DSCHAVEITEMSOLI' => 'NMPESSOADEP', 'DSLABELITEMSOLI' => 'Nome'),
            array('DSCHAVEITEMSOLI' => 'DTNASCIFUNDPARC', 'DSLABELITEMSOLI' => 'Data de nascimento'),
            array('DSCHAVEITEMSOLI' => 'IDSEXOPESSOA', 'DSLABELITEMSOLI' => 'Sexo'),
            array('DSCHAVEITEMSOLI' => 'NRCPFDEP', 'DSLABELITEMSOLI' => 'CPF'),
        );
        $this->_fieldsAlteSituacao = array(
            array('DSCHAVEITEMSOLI' => 'NRSITUFUNCM', 'DSLABELITEMSOLI' => 'Situção funcional'),
            array('DSCHAVEITEMSOLI' => 'DTINISITUFUNC', 'DSLABELITEMSOLI' => 'Data ínicio'),
            array('DSCHAVEITEMSOLI' => 'NRQTDIASITUFUNC', 'DSLABELITEMSOLI' => 'Qtd. de dias'),
            array('DSCHAVEITEMSOLI' => 'DTFIMSITUFUNC', 'DSLABELITEMSOLI' => 'Data fim'),
            array('DSCHAVEITEMSOLI' => 'NRMOTIVOAFASTA', 'DSLABELITEMSOLI' => 'Motivo afastamento'),
            array('DSCHAVEITEMSOLI' => 'NRPROXSITUFUNC', 'DSLABELITEMSOLI' => 'Proxima situação funcional'),
            array('DSCHAVEITEMSOLI' => 'CDTABECDI', 'DSLABELITEMSOLI' => 'Tabela CDI'),
            array('DSCHAVEITEMSOLI' => 'CDDIAGNOST', 'DSLABELITEMSOLI' => 'Doença'),
        );
        $this->_fieldsPlanoSaude = array(
            array('DSCHAVEITEMSOLI' => 'NRBENEFICIOM', 'DSLABELITEMSOLI' => 'Benefício'),
            array('DSCHAVEITEMSOLI' => 'DTINSCBENEFICIARIO', 'DSLABELITEMSOLI' => 'Data Inscrição'),
            array('DSCHAVEITEMSOLI' => 'NRPESSOA', 'DSLABELITEMSOLI' => 'Pessoa'),
            array('DSCHAVEITEMSOLI' => 'IDTPBENEFICIARIO', 'DSLABELITEMSOLI' => 'Tipo Beneficiário'),
        );
        
        $this->_fieldAlteraDados = array(
            array('DSCHAVEITEMSOLI' => 'DSPESSOA', 'DSLABELITEMSOLI' => 'Nome'),
            array('DSCHAVEITEMSOLI' => 'NRCERTCASAPES', 'DSLABELITEMSOLI' => 'Certidao de Casamento'),
            array('DSCHAVEITEMSOLI' => 'NRGRAUINSTR', 'DSLABELITEMSOLI' => 'Grau de Instrução'),
            array('DSCHAVEITEMSOLI' => 'DTCASAPESSOA', 'DSLABELITEMSOLI' => 'Data de casamento'),
            array('DSCHAVEITEMSOLI' => 'NRCARTHABPES', 'DSLABELITEMSOLI' => 'CNH'),
            array('DSCHAVEITEMSOLI' => 'DTHABCARTPES', 'DSLABELITEMSOLI' => 'Dt. Habilitação'),
            array('DSCHAVEITEMSOLI' => 'DSCATEGHABCART', 'DSLABELITEMSOLI' => 'Categoria CNH'),
            array('DSCHAVEITEMSOLI' => 'DTVALHABCARTPES', 'DSLABELITEMSOLI' => 'Validade CNH'),
        );
        
        $this->_fieldAlteraEndereco = array(
            array('DSCHAVEITEMSOLI' => 'NRCEPENDERECO', 'DSLABELITEMSOLI' => 'CEP'),
            array('DSCHAVEITEMSOLI' => 'CDTIPOENDERECO', 'DSLABELITEMSOLI' => 'Tipo de Endereco'),
            array('DSCHAVEITEMSOLI' => 'CDPAIS', 'DSLABELITEMSOLI' => 'Pais'),
            array('DSCHAVEITEMSOLI' => 'SGESTADO', 'DSLABELITEMSOLI' => 'Estado'),
            array('DSCHAVEITEMSOLI' => 'CDMUNICIPIO', 'DSLABELITEMSOLI' => 'Municipio'),
            array('DSCHAVEITEMSOLI' => 'NMBAIRROENDERECO', 'DSLABELITEMSOLI' => 'Bairro'),
            array('DSCHAVEITEMSOLI' => 'CDLOGRADOURO', 'DSLABELITEMSOLI' => 'Logradouro'),
            array('DSCHAVEITEMSOLI' => 'DSENDERECO', 'DSLABELITEMSOLI' => 'Endereco'),
            array('DSCHAVEITEMSOLI' => 'NRIMOVELENDERECO', 'DSLABELITEMSOLI' => 'Numero'),
            array('DSCHAVEITEMSOLI' => 'DSCOMPLEENDERECO', 'DSLABELITEMSOLI' => 'Complemento'),
            array('DSCHAVEITEMSOLI' => 'DSREFERENCIAENDE', 'DSLABELITEMSOLI' => 'Referência'),
        );
        
        $this->_fieldAlteraConta = array(
            array('DSCHAVEITEMSOLI' => 'CDBANCO', 'DSLABELITEMSOLI' => 'Banco'),
            array('DSCHAVEITEMSOLI' => 'CDAGENCIA', 'DSLABELITEMSOLI' => 'Agencia'),
            array('DSCHAVEITEMSOLI' => 'CDCONTCORR', 'DSLABELITEMSOLI' => 'Conta Corrente'),
            array('DSCHAVEITEMSOLI' => 'NRTPCONTCORR', 'DSLABELITEMSOLI' => 'Tipo de Conta Corrente'),
        );
        
        $this->_fieldAlteraComunicacao = array(
            array('DSCHAVEITEMSOLI' => 'CDFORMACOMU', 'DSLABELITEMSOLI' => 'Forma de Comunicação'),
            array('DSCHAVEITEMSOLI' => 'CDPREFIXCOMUPARC', 'DSLABELITEMSOLI' => 'DDD'),
            array('DSCHAVEITEMSOLI' => 'CDCOMUNICAPARC', 'DSLABELITEMSOLI' => 'Descrição (tel, e-mail)'),
        );
        $this->_fieldSelecaoPessoal = array(
            array('DSCHAVEITEMSOLI' => 'NRTEMPLATE', 'DSLABELITEMSOLI' => 'Template'),
            array('DSCHAVEITEMSOLI' => 'NRTIPOPROCESSO', 'DSLABELITEMSOLI' => 'Tipo do Processo'),
            array('DSCHAVEITEMSOLI' => 'NRPARCNEGOCIORESP', 'DSLABELITEMSOLI' => 'Responsavel'),
            array('DSCHAVEITEMSOLI' => 'DSMOTIVO', 'DSLABELITEMSOLI' => 'Motivo'),
            array('DSCHAVEITEMSOLI' => 'LOCALIDADE', 'DSLABELITEMSOLI' => 'Localidade'),
            array('DSCHAVEITEMSOLI' => 'DSDESCRICAO', 'DSLABELITEMSOLI' => 'Descrição'),
            array('DSCHAVEITEMSOLI' => 'DTSOLICITA', 'DSLABELITEMSOLI' => 'Disponível desde:'),
            array('DSCHAVEITEMSOLI' => 'DTLIMSOLICITA', 'DSLABELITEMSOLI' => 'Disponível até:'),
            array('DSCHAVEITEMSOLI' => 'NROCUPACAOM', 'DSLABELITEMSOLI' => 'Ocupação'),
            array('DSCHAVEITEMSOLI' => 'NRESTRUTGEREN', 'DSLABELITEMSOLI' => 'Estrutura Gerencial'),
            array('DSCHAVEITEMSOLI' => 'NRVAGAS', 'DSLABELITEMSOLI' => 'Nº de Vagas'),
            array('DSCHAVEITEMSOLI' => 'IDPUBLICAVAGA', 'DSLABELITEMSOLI' => 'Publicar Vaga'),
            array('DSCHAVEITEMSOLI' => 'NMPROCESSO', 'DSLABELITEMSOLI' => 'Nome do Processo'),
        );
        $this->_fieldAlteraEscala = array(
            array('DSCHAVEITEMSOLI' => 'NRESCALATRABM', 'DSLABELITEMSOLI' => 'Escala de Trabalho'),
            array('DSCHAVEITEMSOLI' => 'NRTURNO', 'DSLABELITEMSOLI' => 'Turno'),
            array('DSCHAVEITEMSOLI' => 'DTINIESCALA', 'DSLABELITEMSOLI' => 'Inicio da escala'),
            array('DSCHAVEITEMSOLI' => 'NRMOTIVOESCALA', 'DSLABELITEMSOLI' => 'Motivo'),
        );
        
        $this->_fieldsDesligamento = array(
            array('DSCHAVEITEMSOLI' => 'NRTPAVIPRE', 'DSLABELITEMSOLI' => 'Tipo de Aviso Previo'),
            array('DSCHAVEITEMSOLI' => 'NRTPDEMISSAO', 'DSLABELITEMSOLI' => 'Tipo de Demissão'),
            array('DSCHAVEITEMSOLI' => 'NRMOTIVORESC', 'DSLABELITEMSOLI' => 'Motivo da Rescisão'),
            array('DSCHAVEITEMSOLI' => 'DTAVISOPREVIO', 'DSLABELITEMSOLI' => 'Data de Aviso Prévio'),
            array('DSCHAVEITEMSOLI' => 'DTRESCISAOVINC', 'DSLABELITEMSOLI' => 'Data de Rescisão'),
            );
        
}





    public function retornaVinculoFromOperador($nrorg, $cdoperador)
    {
        return $this->solicitacaoRepository->retornaVinculoFromOperador($nrorg, $cdoperador, $this->hcmParameters->getDtCompetencia());
    }
    
    public function validaSetPerfilacesso($nrorg,$tiposoli,$nrperfacesso = null){
        $perfis = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findBy(array("nrorg" => $nrorg, "nrtiposolicitacao" => $tiposoli));
        $perfisAt = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findBy(array("nrorg" => $nrorg, "nrtiposolicitacao" => $tiposoli,"nrperfacestiposoli"=>$nrperfacesso));
        if(empty($perfis))
            return true;
        else
            if(empty($perfisAt))
                return false;
            else 
                return true;
    }


    public function validaPermissaoOperador($rowSolicitacao) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $erro["Bloquear"] = false;
        
        if(!$this->validaPerfildeAcesso($rowSolicitacao['NRTIPOSOLICITACAO'])) {
            $erro["Texto"] = "Não existe perfil de acesso para este tipo de solicitação.";
            $erro["Bloquear"] = true;
        }
        
        $vincOper = $this->retornaVinculoFromOperador($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getCdoperador());
        
        foreach($vincOper as $vinc){
            if($vinc['NRVINCULOM'] == $rowSolicitacao['NRVINCULOM'])
            return $erro;
        }
        $acesso = $this->solicitacaoRepository->verificaAcessoOperVinculo($this->hcmParameters->getNrorgTrabalho(), $rowSolicitacao['NRVINCULOM'], $this->hcmParameters->getCdoperador(), $rowSolicitacao['NRTIPOSOLICITACAO']);
        if (!$acesso) {
            $erro["Texto"] = "Operador não possui perfil de acesso.";
            $erro["Bloquear"] = true;
        }
        
        
        return $erro;
    }
    
    public function validaPerfildeAcesso($tiposoli){
        $perfil = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findOneBy(array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(),'nrtiposolicitacao'=>$tiposoli));
        if(!$perfil)    
            return false;
        return true;
    }
    
    public function insereSolicitacaoAdm($dadosSoli)
    {
        try {
            
            if (!is_null($dadosSoli)) {
                
                try {
                    $dadosSoli['NRSOLICITACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_SOLICITACAO', 1, 12, true);
                    $dadosSoli['DSOBSSOLICITACAO'] = isset($dadosSoli['DSOBSSOLICITACAO'])?$dadosSoli['DSOBSSOLICITACAO']:null;
                    $this->entityManager->beginTransaction();
                    $this->solicitacaoRepository->insereSolicitacao(
                        $dadosSoli['NRSOLICITACAO'], $dadosSoli['NRTIPOSOLICITACAO'], $dadosSoli['NRVINCULOM'], $dadosSoli['DSOBSSOLICITACAO']
                    );
                    
                        $nritemsolicitacao = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ITEMSOLICITACAO', 1, 12, true);
                        $this->solicitacaoRepository->insereItemSolicitacao(
                            $nritemsolicitacao, $dadosSoli['NRSOLICITACAO'], 'DTMESCOMPETENC', 'Competencia', $dadosSoli['DTMESCOMPETENC']->format('d/m/Y')
                        );
                    $this->entityManager->commit();
                } catch (\Exception $e) {
                    $this->entityManager->rollback();
                    throw $e;
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $rowSolicitacao
     * @throws \Exception
     */
    public function executaInsertSolicitacao($rowSolicitacao) {
        try {
            $erro = $this->validaPermissaoOperador($rowSolicitacao);
            if(!$erro["Bloquear"]) {
                $dadosSoli = $this->preparaDados($rowSolicitacao);
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                if (!is_null($dadosSoli)) {
                    try {
                        $this->entityManager->beginTransaction();
                        $this->solicitacaoRepository->insereSolicitacao(
                            $dadosSoli['NRSOLICITACAO'], $dadosSoli['NRTIPOSOLICITACAO'], $dadosSoli['NRVINCULOM'], $dadosSoli['DSOBSSOLICITACAO']
                        );
                        foreach ($dadosSoli['ITENS'] as $itemSoli) {
                            $nritemsolicitacao = \Geral\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 22, false);
                            $this->solicitacaoRepository->insereItemSolicitacao(
                                $nritemsolicitacao, $dadosSoli['NRSOLICITACAO'], $itemSoli['DSCHAVEITEMSOLI'], $itemSoli['DSLABELITEMSOLI'], $itemSoli['DSVALORITEMSOLI']
                            );
                        }
                        $this->entityManager->commit();
                        return true;
                    } catch (\Exception $e) {
                        $this->entityManager->rollback();
                        throw $e;
                    }
                }
            } else return $erro;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $rowSolicitacao
     * @return array
     * @throws \Exception
     */
    public function preparaDados($rowSolicitacao)
    {
        try {
            $dadosSoli = array();
            $itens = array();

            $dadosSoli['NRSOLICITACAO'] = $rowSolicitacao['NRSOLICITACAO'];
            $dadosSoli['NRTIPOSOLICITACAO'] = $rowSolicitacao['NRTIPOSOLICITACAO'];
            $dadosSoli['NRVINCULOM'] = $rowSolicitacao['NRVINCULOM'];
            $dadosSoli['DSOBSSOLICITACAO'] = $rowSolicitacao['DSOBSSOLICITACAO'];

            switch ($rowSolicitacao['NRTIPOSOLICITACAO']) {
                case self::TP_SOLI_FERIAS:
                    $itens = $this->preparaItens($this->_fieldsFerias, $rowSolicitacao);
                    break;
                case self::TP_SOLI_SALARIO:
                    $itens = $this->preparaItens($this->_fieldsSalario, $rowSolicitacao);
                    break;
                case self::TP_SOLI_OCUPACAO:
                    $itens = $this->preparaItens($this->_fieldsOcupacao, $rowSolicitacao);
                    break;
                case self::TP_SOLI_TREINAMENTO:
                    $itens = $this->preparaItens($this->_fieldsTreinamento, $rowSolicitacao);
                    break;
                case self::TP_SOLI_VTRANS:
                    $itens = $this->preparaItens($this->_fieldsValeTransporte, $rowSolicitacao);
                    break;
                case self::TP_SOLI_MOVIM:
                    $itens = $this->preparaItens($this->_fieldsMovimentacao, $rowSolicitacao);
                    break;
                case self::TP_SOLI_DEP:
                    $itens = $this->preparaItens($this->_fieldsDependente, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTESITU:
                    $itens = $this->preparaItens($this->_fieldsAlteSituacao, $rowSolicitacao);
                    break;
                case self::TP_SOLI_PLANOSAUDE:
                    $itens = $this->preparaItens($this->_fieldsPlanoSaude, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTERADADOS:
                    $itens = $this->preparaItens($this->_fieldAlteraDados, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTERAENDERECO:
                    $itens = $this->preparaItens($this->_fieldAlteraEndereco, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTERACONTA:
                    $itens = $this->preparaItens($this->_fieldAlteraConta, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTERACOMU:
                    $itens = $this->preparaItens($this->_fieldAlteraComunicacao, $rowSolicitacao);
                    break;
                case self::TP_SOLI_PESSOAL:
                    $itens = $this->preparaItens($this->_fieldSelecaoPessoal, $rowSolicitacao);
                    break;
                case self::TP_SOLI_ALTERAESCALA:
                    $itens = $this->preparaItens($this->_fieldAlteraEscala, $rowSolicitacao);
                    break;
                case self::TP_SOLI_DESLIGAMENTO:
                    $itens = $this->preparaItens($this->_fieldsDesligamento, $rowSolicitacao);
                    break;
            }
            $dadosSoli['ITENS'] = $itens;

            return $dadosSoli;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $arrayFields
     * @param $rowSolicitacao
     * @return array
     * @throws \Exception
     */
    public function preparaItens($arrayFields, $rowSolicitacao)
    {
        try {
            $itens = array();
            foreach ($arrayFields as $field) {
                if(array_key_exists($field['DSCHAVEITEMSOLI'],$rowSolicitacao))
                    if(isset($rowSolicitacao[$field['DSCHAVEITEMSOLI']]) && $rowSolicitacao[$field['DSCHAVEITEMSOLI']] != "" )
                        $itens[] = array(
                            'DSCHAVEITEMSOLI' => $field['DSCHAVEITEMSOLI'],
                            'DSLABELITEMSOLI' => $field['DSLABELITEMSOLI'],
                            'DSVALORITEMSOLI' => $rowSolicitacao[$field['DSCHAVEITEMSOLI']],
                        );
            }
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        } 
    }

    public function processarSolicitacaoAprovada($rowSolicitacao)
    {
        try {
            $itensSolicitacao = $this->solicitacaoRepository->retornaItensSolicitacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRSOLICITACAO']);
            switch ($rowSolicitacao['NRTIPOSOLICITACAO']) {
                case self::TP_SOLI_FERIAS:
                    $this->insereGozoFerias($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'], $itensSolicitacao);
                    break;
                case self::TP_SOLI_MOVIM:
                    $this->insereMovimentacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'], $rowSolicitacao['NRTPESTRUTURAMLEGAL'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_SALARIO:
                    $this->insereAlteracaoSalarial($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'], $itensSolicitacao);
                    break;
                case self::TP_SOLI_OCUPACAO:
                    $this->insereAlteracaoOcupacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'], $rowSolicitacao['DSOBSSOLICITACAO'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_TREINAMENTO:
                    $this->insereTreinamento($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_VTRANS:
                    $this->insereValeTransporte($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'], $rowSolicitacao['DSOBSSOLICITACAO'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_DEP:
                    $this->insereDependente($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTESITU:
                    $this->insereAlteraSituacaoFunc($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_PLANOSAUDE:
                    $itens = $this->insereBeneficiario($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTERADADOS:
                    $itens = $this->alteraDadosPessoa($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTERAENDERECO:
                    $itens = $this->alteraEndereco($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTERACOMU:
                    $itens = $this->alteraComunicacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTERACONTA:
                    $itens = $this->alteraConta($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_PESSOAL:
                    $itens = $this->insereProcesso($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$rowSolicitacao['IDSTATUS'], $itensSolicitacao);
                    break;
                case self::TP_SOLI_ALTERAESCALA:
                    $itens = $this->alteraEscala($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_DESLIGAMENTO:
                    $itens = $this->realizaDemissao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                case self::TP_SOLI_ADMISSAO:
                    $itens = $this->realizaAdmissao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM'],$itensSolicitacao);
                    break;
                
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function processarSolicitacaoReprovada($rowSolicitacao)
    {
        try {
            $itensSolicitacao = $this->solicitacaoRepository->retornaItensSolicitacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRSOLICITACAO']);
            switch ($rowSolicitacao['NRTIPOSOLICITACAO']) {
                case self::TP_SOLI_ADMISSAO:
                    $itens = $this->reprovaAdmissao($rowSolicitacao['NRORG'], $rowSolicitacao['NRVINCULOM']);
                    break;   
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereGozoFerias($nrorg, $nrvinculom, $itens)
    {
        try {
            foreach ($itens as $campo) {
                $idadiant13salar = "N";
                $abonoPecuniario = "N";
                switch ($campo['DSCHAVEITEMSOLI']) {
                    case 'DTINIGOZOFERIAS':
                        $dtinigozoferias = $campo['DSVALORITEMSOLI'] ? DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO,true) : null;
                        break;
                    case 'QTDEDIASFERIAS':
                        $qtdediasferias = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTFIMGOZOFERIAS':
                        $dtfimgozoferias = $campo['DSVALORITEMSOLI'] ? DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO,true) : null;
                        break;
                    case 'DTRETORNOFERIAS':
                        $dtretornoferias = $campo['DSVALORITEMSOLI'] ? DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO,true) : null;
                        break;
                    case 'IDADIANT13SALAR':
                        $idadiant13salar = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'ABONOPECUNIARIO':
                        $abonoPecuniario = $campo['DSVALORITEMSOLI'];
                        break;
                }
            }
            $ferias = $this->gozoFeriasRepository->retornaFeriasEmAberto($nrorg, $nrvinculom, $this->hcmParameters->getDtCompetencia());
            /** @var \Teknisa\Model\Gestaopessoas\Entities\FpaGozoferias $gozoFerias */
            $gozoFerias = new $this->fpaGozoferias;
            if(!empty($ferias)){
                $gozoFerias->setNrorg($nrorg);
                $gozoFerias->setNrferias($ferias[0]['NRFERIAS']);
                $gozoFerias->setDtinigozoferias($dtinigozoferias);
                $gozoFerias->setDtfimgozoferias($dtfimgozoferias);
                $gozoFerias->setDtretornoferias($dtretornoferias);
                $gozoFerias->setIdadiant13salar($idadiant13salar);
                $gozoFerias->setQtdiasabonoferi($valor = (($abonoPecuniario == "S") ? 10 : 0));
                $gozoFerias->setDtinclusao(DateUtil::getDataAtual(false));
                $gozoFerias->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $gozoFerias->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $gozoFerias->setNrgozoferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_GOZOFERIAS', 1, 12, true));
                $this->entityManager->persist($gozoFerias);
                $this->entityManager->flush();
            }else{
                throw new \Exception("Registro de ferias não encontrado para o vinculo");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereAlteracaoSalarial($nrorg, $nrvinculom, $itens)
    {
        try {
            foreach ($itens as $campo) {
                switch ($campo['DSCHAVEITEMSOLI']) {
                    case 'DTALTESALARIO':
                        $dtaltesalario = $campo['DSVALORITEMSOLI'] ? DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), 'd/m/Y', true) : null;
                        break;
                    case 'NRTIPOALTERA':
                        $nrtipoaltera= $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRTPMODALIDSAL':
                        $nrtpmodalidsal = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'IDTPSALARIO':
                        $idtpsalario = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NOVOVALORSALARIO':
                        $novoVlrSalario = $campo['DSVALORITEMSOLI'];
                        break;
                }
            }
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltesalario $alteSalario */
            $alteSalario = new $this->gpeAltesalrio;
            $alteSalario->setNrorg($nrorg);
            $alteSalario->setNrvinculom($nrvinculom);
            $alteSalario->setDtaltesalario($dtaltesalario);
            $alteSalario->setNrocorrencia(1);
            $alteSalario->setNrtipoaltera($nrtipoaltera);
            $alteSalario->setNrtpmodalidsal($nrtpmodalidsal);
            $alteSalario->setIdtpsalario($idtpsalario);
            $alteSalario->setVrsalario($novoVlrSalario);
            $alteSalario->setDtinclusao(DateUtil::getDataAtual(false));
            $alteSalario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $alteSalario->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $alteSalario->setNraltesalario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true));
            $this->entityManager->persist($alteSalario);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insereTreinamento($nrorg, $nrvinculom, $itens)
    {
        try {
            foreach ($itens as $campo) {
                switch ($campo['DSCHAVEITEMSOLI']) {
                    case 'NRCURSO':
                        $nrcurso = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRPERFACESAPROV':
                        $nrperfacesaprov = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRMETODOTRE':
                        $nrmetodotre = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NMTREINAMENTO':
                        $nmtreinamento = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRPERFACESEXEC':
                        $nrperfacesexec = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'IDTREINAMENTOINT':
                        $idtreinamentoint = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTINIPLANEJADOTRE':
                        $dtiniplanejadotre = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), 'd/m/Y', true);
                        break;
                    case 'DTFIMPLANEJADOTRE':
                        $dtfimplanejadotre = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), 'd/m/Y', true);
                        break;
                }
            }
            
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));
            
            $treinamento = new $this->gpeTreinamento;
            
            $treinamento->setNrparcnegaprov($result->getNrparcnegocio());
            $treinamento->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $treinamento->setDtfimplanejadotre($dtfimplanejadotre);
            $treinamento->setDtinclusao(DateUtil::getDataAtual());
            $treinamento->setDtiniplanejadotre($dtiniplanejadotre);

            $treinamento->setIdeficaciatreinamento('N');
            $treinamento->setIdstatustreinamento('PENDENTE');
            $treinamento->setIdtreinamentoint($idtreinamentoint);
            
            $treinamento->setNmtreinamento($nmtreinamento);
            $treinamento->setNrcurso($nrcurso);
            $treinamento->setNrmetodotre($nrmetodotre);
            $treinamento->setNrorg($nrorg);
            $treinamento->setNrorginclusao($nrorg);

            $treinamento->setNrperfacesaprov($nrperfacesaprov);
            $treinamento->setNrperfacesexec($nrperfacesexec);

            $treinamento->setNrtreinamento(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_TREINAMENTO', 1, 12, true));
            $this->entityManager->persist($treinamento);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insereAlteracaoOcupacao($nrorg, $nrvinculom , $dsobssolicitacao, $itens)
    {
        try {
            $nrmotivoocupa = null;
            $nrtpmodalidsal = null;
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
            
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NROCUPACAOM_OCU':
                        $nrocupacaom_ocu = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRMOTIVOOCUPA':
                        $nrmotivoocupa = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRTPMODALIDSAL':
                        $nrtpmodalidsal = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRTIPOOCUPACAO':
                        $nrtipoocupacao = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTINIOCUPACAO':
                        $dtiniocupacao = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                }
            }
            
            $result = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtiniocupacao" => $dtiniocupacao));

            if($result)
                throw new \Exception("Já existe uma alteração de ocupação para este vinculo nesta data");
            $this->solicitacaoRepository->atualizaAlteOcupa($nrorg,$nrvinculom,DateUtil::subtraiIntervalo($dtiniocupacao,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),$this->hcmParameters->getCdoperador());
            
            $alteOcupacao = new $this->gpeAlteocupacao;
            $alteOcupacao->setNrorg($nrorg);
            $alteOcupacao->setNrvinculom($nrvinculom);
            $alteOcupacao->setDsobservacao($dsobssolicitacao);
            $alteOcupacao->setDtiniocupacao($dtiniocupacao);
            $alteOcupacao->setNrmotivoocupa($nrmotivoocupa);
            $alteOcupacao->setNrocupacaom($nrocupacaom_ocu);
            $alteOcupacao->setNrtipoocupacao($nrtipoocupacao);
            $alteOcupacao->setNrtpmodalidsal($nrtpmodalidsal);
            $alteOcupacao->setDtinclusao(DateUtil::getDataAtual(false));
            $alteOcupacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $alteOcupacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $alteOcupacao->setNralteocupacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true));
            $this->entityManager->persist($alteOcupacao);
            $this->entityManager->flush();
            
            $vinc = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
            if($vinc->getDtmescompetenc() == $dtcompetencia){
                $vinc->setDtultatu(DateUtil::getDataAtual());
                $vinc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $vinc->setCdoperultatu($this->hcmParameters->getCdoperador());
                $vinc->setNrcargo($nrocupacaom_ocu);
                $this->entityManager->persist($vinc);
                $this->entityManager->flush();
            }
            else
            $this->vinculoServices->novoHistorico($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $nrocupacaom_ocu, $vinc->getNrescalatrabm(),
                                  $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
                                  $vinc->getIdtppagamento(), $vinc->getNrestrutlegal(), $vinc->getNrestrutgeren(), $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
                                  $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
                                  $vinc->getNrestrutsind(), $vinc->getConfidencial());
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insereValeTransporte($nrorg, $nrvinculom , $dsobssolicitacao, $itens){
        try {
            $dtfimlinhavinc = null;
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NRLINHATRANSP':
                        $nrlinhatransp = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRTIPOLINHAVINC':
                        $nrtipolinhavinc = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRSENTIDOLINHA':
                        $nrsentidolinha = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'QTLINHATARIFA':
                        $qtlinhatarifa = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTINILINHAVINC':
                        $dtinilinhavinc = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                    case 'DTFIMLINHAVINC':
                        $dtfimlinhavinc = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                }
            }
            
            $vabLinhavinc = new $this->vabLinhavinc;
            $vabLinhavinc->setNrorg($nrorg);
            $vabLinhavinc->setNrvinculom($nrvinculom);
            $vabLinhavinc->setNrsentidolinha($nrsentidolinha);
            $vabLinhavinc->setQtlinhatarifa($qtlinhatarifa);
            $vabLinhavinc->setDtinilinhavinc($dtinilinhavinc);
            $vabLinhavinc->setDtfimlinhavinc($dtfimlinhavinc);
            $vabLinhavinc->setNrlinhatarifa($nrlinhatransp);
            $vabLinhavinc->setNrtipolinhavinc($nrtipolinhavinc);
            $vabLinhavinc->setDsobslinhavinc($dsobssolicitacao);
            $vabLinhavinc->setDtinclusao(DateUtil::getDataAtual(false));
            $vabLinhavinc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $vabLinhavinc->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vabLinhavinc->setNrlinhavinc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_LINHAVINC', 1, 12, true));
            $this->entityManager->persist($vabLinhavinc);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }
    
    public function insereMovimentacao($nrorg, $nrvinculom , $nrtipoestrutlegal, $itens)
    {
        try {
            $nrmotivotransf = null;
            $dtfimmoviment = null;
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
            
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NRTIPOTRANSFER':
                        $nrtipotransfer = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRTPMOVTRANSFM':
                        $nrtpmovtransfm = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRMOTIVOTRANSF':
                        $nrmotivotransf = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRESTRUTURAM':
                        $nrestruturam = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTINIMOVIMENT':
                        $dtinimoviment = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                    case 'DTFIMMOVIMENT':
                        $dtfimmoviment = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                }
            }
        
            $estrutura_nova = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestruturam));
            $result = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtinimoviment" => $dtinimoviment));

            if($result){
                $estrutura_historico = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findBy(array("nrorg"=> $nrorg,"nrestruturam" => $result->getNrestruturam()));
                if($estrutura_historico->getNrtipoestrutura() == $estrutura_nova->getNrtipoestrutura())
                    throw new \Exception("Já existe uma movimentação para este tipo de estrutura nesta data");
            }
            if($nrtipotransfer == 1){
                if($estrutura_nova->getNrtipoestrutura() != $nrtipoestrutlegal && $estrutura_nova->getNrtipoestrutura() != 10)
                    throw new \Exception("A nova estrutura legal deve ser do mesmo tipo que a antiga.");
            }
            else if($estrutura_nova->getNrtipoestrutura() == $nrtipoestrutlegal)
                    throw new \Exception("A nova estrutura legal deve ser de tipo diferente que a antiga.");
            
            if(DateUtil::comparaDatas($dtcompetencia,$dtinimoviment, DateUtil::DEPOIS)){
                    throw new \Exception("A movimentação não pode ser retroativa");
            }

            //throw new \Exception("Já existe uma alteração de ocupação para este vinculo nesta data");
            
            $movimentacao = new $this->gpeMovimentacao;
            $movimentacao->setNrorg($nrorg);
            $movimentacao->setNrvinculom($nrvinculom);
            $movimentacao->setNrtipoestrutura($estrutura_nova->getNrtipoestrutura());
            $movimentacao->setNrestruturam($nrestruturam);
            $movimentacao->setNrmotivotransf($nrmotivotransf);
            $movimentacao->setNrtipotransfer($nrtipotransfer);
            $movimentacao->setNrtpmovtransfm($nrtpmovtransfm);
            $movimentacao->setDtfimmoviment($dtfimmoviment);
            $movimentacao->setDtinimoviment($dtinimoviment);
            $movimentacao->setDtinclusao(DateUtil::getDataAtual(false));
            $movimentacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $movimentacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $movimentacao->setNrmovimentacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true));
            $this->entityManager->persist($movimentacao);
            $this->entityManager->flush();
            
            $vinc = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
            if($vinc->getDtmescompetenc() == $dtcompetencia){
                $vinc->setDtultatu(DateUtil::getDataAtual());
                $vinc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $vinc->setCdoperultatu($this->hcmParameters->getCdoperador());
                if($nrtipotransfer == 1)
                    $vinc->setNrestrutlegal($nrestruturam);
                else 
                    $vinc->setNrestrutgeren($nrestruturam);
                $this->entityManager->persist($vinc);
                $this->entityManager->flush();
            }
            else
            $this->vinculoServices->novoHistorico($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $vinc->getNrcargo(), $vinc->getNrescalatrabm(),
                                  $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
                                  $vinc->getIdtppagamento(), (($nrtipotransfer == 1)?$nrestruturam:$vinc->getNrestrutlegal()),(($nrtipotransfer == 1)?$vinc->getNrestrutgeren():$nrestruturam) , $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
                                  $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
                                  $vinc->getNrestrutsind(), $vinc->getConfidencial());
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insereDependente($nrorg, $nrvinculom , $itens){
        try {
            $dtnascifundparc = null;
            $nrcpfdep = null;
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                case 'NRTIPORELACIONA':
                    $nrtiporelaciona = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'NMPESSOADEP':
                    $nmpessoa = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'IDSEXOPESSOA':
                    $idsexopessoa = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'DTNASCIFUNDPARC':
                    $dtnascifundparc = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                    break;
                case 'NRCPFDEP':
                    $nrcpfdep = str_replace(array('.','/','-'),'', $campo['DSVALORITEMSOLI']);
                    break;
                }
            }
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $this->dependenteVincService->realizaCadastroPessoaDependente($nrvinculom, $vinculom->getNrpessoa(), $nmpessoa, null, null, null, null,
                    null, $idsexopessoa, $dtnascifundparc, $nrtiporelaciona, null, $nrcpfdep, null);

        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    public function insereAlteraSituacaoFunc($nrorg, $nrvinculom , $itens){
        try {
            $nrqtdiasitufunc = null;
            $nrproxsitufunc = null;
            $cdtabecdi = null;
            $nrmotivoafasta = null;
            $cddiagnost = null;
            $dtfimsitufunc = null;
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                case 'NRSITUFUNCM':
                    $nrsitufuncm = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'NRQTDIASITUFUNC':
                    $nrqtdiasitufunc = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'NRPROXSITUFUNC':
                    $nrproxsitufunc = $campo['DSVALORITEMSOLI']; 
                    break;
                case 'CDTABECDI':
                    $cdtabecdi = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'NRMOTIVOAFASTA':
                    $nrmotivoafasta = $campo['DSVALORITEMSOLI'];
                    break;    
                case 'CDDIAGNOST':
                    $cddiagnost = $campo['DSVALORITEMSOLI'];
                    break;    
                case 'DTINISITUFUNC':
                    $dtinisitufunc = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                    break;
                case 'DTFIMSITUFUNC':
                    $dtfimsitufunc = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                    break;
                }
            }
           
                
            $this->alteSituacaoFuncionalService->validaExisteDataSituFunc($nrorg, $nrvinculom, $dtinisitufunc->format('d/m/Y'));
                
            // Valida a quantidade de dias de afastamento para mesmo CID
            $retorno = $this->alteSituacaoFuncionalService->verificAfastMesmoCid(
                $nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc, $dtfimsitufunc, $nrqtdiasitufunc, $cdtabecdi
            );
            
            if (is_array($retorno)) {
                if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO) {
                    $nrqtdiasitufunc = $retorno['NRQTDIASITUFUNC'];
                    $dtfimsitufunc = $retorno['DTFIMSITUFUNC'];
                }
                else if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MENOR_15) {
                    $nrsitufuncm = $retorno['NRSITUFUNCM'];
                }
            }

            $this->alteSituacaoFuncionalService->beforeSave($nrorg, $nrvinculom, $this->hcmParameters->getDtCompetencia(), $dtinisitufunc->format('d/m/Y'));

            $nraltesitufunc = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true);
            
            $altesitufunc = new $this->alteSituFunc;
            $altesitufunc->setCddiagnost($cddiagnost);
            $altesitufunc->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $altesitufunc->setCdtabecdi($cdtabecdi);
            $altesitufunc->setDtestabsitufunc();
            $altesitufunc->setDtfimsitufunc($dtfimsitufunc);
            $altesitufunc->setDtinclusao(DateUtil::getDataAtual());
            $altesitufunc->setDtinisitufunc($dtinisitufunc);
            $altesitufunc->setNraltesitufunc($nraltesitufunc);
            $altesitufunc->setNrmotivoafasta($nrmotivoafasta);
            $altesitufunc->setNrorg($nrorg);
            $altesitufunc->setNrorginclusao($nrorg);
            $altesitufunc->setNrproxsitufunc($nrproxsitufunc);
            $altesitufunc->setNrqtdiasitufunc($nrqtdiasitufunc);
            $altesitufunc->setNrsitufuncm($nrsitufuncm);
            $altesitufunc->setNrvinculom($nrvinculom);
            $this->entityManager->persist($altesitufunc);
            $this->entityManager->flush();
            

            $this->alteSituacaoFuncionalService->atualizaSitufuncHistoricoVinculo($nrorg, $nrvinculom, $nrsitufuncm);

            $retorno = $this->alteSituacaoFuncionalService->geraEstabilidadeSituFunc($nrorg, $nrvinculom, $nrsitufuncm, $dtinisitufunc, $nrqtdiasitufunc, $dtfimsitufunc, false);

            $this->alteSituacaoFuncionalService->verifDtUltSitufunc($nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc->format('d/m/Y'));
            if ($nrproxsitufunc) {
                $proxsituacao = $this->alteSituacaoFuncionalService->verifProxSitufunc($nrorg, $nrvinculom, $dtinisitufunc->format('d/m/Y'), $dtfimsitufunc->format('d/m/Y'), $nrproxsitufunc);
               
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    public function insereBeneficiario($nrorg, $nrvinculom , $itens){
        try {
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                case 'NRBENEFICIOM':
                    $nrbeneficiom = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'NRPESSOA':
                    $nrpessoa = $campo['DSVALORITEMSOLI']; 
                    break;    
                case 'IDTPBENEFICIARIO':
                    $idtpbeneficiario = $campo['DSVALORITEMSOLI'];
                    break;    
                case 'DTINSCBENEFICIARIO':
                    $dtinscbeneficiario = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                    break;
                }
            }
            
            $result = $this->entityManager->getRepository(Repositories::VAB_BENEFICIARIO)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"nrpessoa" => $nrpessoa,"nrbeneficiom" => $nrbeneficiom));
            if($result)
                throw new \Exception(Messages::HCM02000_VAB_BENEFICIARIO_JA_CADASTRADO_ERROR);

            $beficiario = new $this->beneficiario;
            $beficiario->setNrorg($nrorg);
            $beficiario->setNrvinculom($nrvinculom);
            $beficiario->setNrbeneficiom($nrbeneficiom);
            $beficiario->setNrpessoa($nrpessoa);
            $beficiario->setIdtpbeneficiario($idtpbeneficiario);
            $beficiario->setDtinscbeneficiario($dtinscbeneficiario);
            $beficiario->setQtbeneficio(1);
            $beficiario->setDtinclusao(DateUtil::getDataAtual(false));
            $beficiario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $beficiario->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $beficiario->setNrbeneficiario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true));
            $this->entityManager->persist($beficiario);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    public function alteraDadosPessoa($nrorg,$nrvinculom,$itens){
    try {   
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
            
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $vinculom->getNrpessoa()));

            $pessoaHistorico = $this->pessoaRepository->retornaHistoricoAtualPorPessoa($nrorg,$result->getNrpessoa(),$dtcompetencia);
            
            $nrcertcasapes = $pessoaHistorico->getNrcertcasapes();
            $nrgrauinstr = $pessoaHistorico->getNrgrauinstr();
            $nrcarthabpes = $pessoaHistorico->getNrcarthabpes();
            $dscateghabcart = $pessoaHistorico->getDscateghabcart();
            $dtcasapessoa = $pessoaHistorico->getDtcasapessoa();
            $dthabcartpes = $pessoaHistorico->getDthabcartpes();
            $dtvalhabcartpes = $pessoaHistorico->getDtvalhabcartpes();
            
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NRCERTCASAPES':
                        $nrcertcasapes = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRGRAUINSTR':
                        $nrgrauinstr = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DSPESSOA':
                    case 'NMPESSOA':
                        $dspessoa = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRCARTHABPES':
                        $nrcarthabpes = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DSCATEGHABCART':
                        $dscateghabcart = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DTCASAPESSOA':
                        $dtcasapessoa = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                    case 'DTHABCARTPES':
                        $dthabcartpes = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                    case 'DTVALHABCARTPES':
                        $dtvalhabcartpes = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                }
            }


            if($pessoaHistorico->getDtmescompetenc() == $dtcompetencia){
                
                $pessoaHistorico->setNmpessoa($dspessoa);
                $pessoaHistorico->setNrcertcasapes($nrcertcasapes);
                $pessoaHistorico->setNrgrauinstr($nrgrauinstr);
                $pessoaHistorico->setNrcarthabpes($nrcarthabpes);
                $pessoaHistorico->setDscateghabcart($dscateghabcart);
                $pessoaHistorico->setDtcasapessoa($dtcasapessoa);
                $pessoaHistorico->setDthabcartpes($dthabcartpes);
                $pessoaHistorico->setDtvalhabcartpes($dtvalhabcartpes);
                
                $this->entityManager->persist($pessoaHistorico);
                $this->entityManager->flush();
            }
            else{
                $this->pessoaService->novoHistorico(
                    $nrorg,$pessoaHistorico->getNrpessoa(),$dtcompetencia,$dspessoa,$pessoaHistorico->getDsapelidopess(),$pessoaHistorico->getNrcertnascpes(),$pessoaHistorico->getDtnascpessoa(),
                    $pessoaHistorico->getIdsexopessoa(),$pessoaHistorico->getNrcondfispes(),$pessoaHistorico->getNrcpfpessoa(),$pessoaHistorico->getNrctpspessoa(),$pessoaHistorico->getNrseriectpspes(),$pessoaHistorico->getSgufctpspes(),
                    $pessoaHistorico->getDtctpspessoa(),$pessoaHistorico->getNrpispaseppes(),$pessoaHistorico->getDtinscpispasep(),$pessoaHistorico->getNrinscinsspes(),$pessoaHistorico->getNrinscisspes(),$pessoaHistorico->getCdestacivil(),
                    $nrcertcasapes,$dtcasapessoa,$pessoaHistorico->getIdisentituelei(),$pessoaHistorico->getNrtitueleipes(),$pessoaHistorico->getNrsecaeleipes(),$pessoaHistorico->getNrzonaeleipes(),
                    $nrcarthabpes,$dthabcartpes,$dtvalhabcartpes,$dscateghabcart,$pessoaHistorico->getNrcatmilitarpe(),$pessoaHistorico->getNrcertresepes(),
                    $pessoaHistorico->getDscategcertrese(),$pessoaHistorico->getDtcertresepes(),$pessoaHistorico->getCdexcertresepes(),$pessoaHistorico->getNrracapessoa(),$pessoaHistorico->getIdgrupsangupes(),$pessoaHistorico->getIdfatorrhpes(),
                    $pessoaHistorico->getCdpais(),$pessoaHistorico->getSgestado(),$pessoaHistorico->getCdmunicipio(),$pessoaHistorico->getNrrgpessoa(),$pessoaHistorico->getCdexrgpessoa(),$pessoaHistorico->getSgufrgpessoa(),
                    $pessoaHistorico->getDslocalexrgpes(),$pessoaHistorico->getDtexrgpessoa(),$pessoaHistorico->getNrnacionalid(),$nrgrauinstr,$pessoaHistorico->getDspronometrat() ,$pessoaHistorico->getNranochegapais() ,
                    $pessoaHistorico->getNrdocuestrang(),$pessoaHistorico->getIdpesnaturaliza(),$pessoaHistorico->getDtnaturalizapes(),$pessoaHistorico->getIdposvistoperm(),$pessoaHistorico->getDtpermavisto(),$pessoaHistorico->getNrconselhoreg(),
                    $pessoaHistorico->getNrinscrconsreg(),$pessoaHistorico->getDtexconsreg(),$pessoaHistorico->getSgufconsreg(),$pessoaHistorico->getCdtpbaixapag(),$pessoaHistorico->getNrlivrocertnasc(),
                    $pessoaHistorico->getNrfolhacertnasc(), $pessoaHistorico->getDscomplcertnasc(),$pessoaHistorico->getNrregctpspes()
                    );    
            }
            
            if($vinculom->getNmvinculom() != $dspessoa){
                $vinculom->setNmvinculom($dspessoa);
                $this->entityManager->persist($vinculom);
                $this->entityManager->flush();
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function alteraEndereco($nrorg,$nrvinculom,$itens){
    try {   
                $nrcependereco = null;
                $cdtipoendereco = null;
                $cdpais = null;
                $sgestado = null;
                $cdmunicipio = null;
                $nmbairroendereco = null;
                $cdlogradouro = null;
                $dsendereco = null;
                $nrimovelendereco = null;
                $dscompleendereco = null;
                $dsreferenciaende = null;
                
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NRCEPENDERECO':
                        $nrcependereco = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDTIPOENDERECO':
                        $cdtipoendereco = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDPAIS':
                        $cdpais = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'SGESTADO':
                        $sgestado = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDMUNICIPIO':
                        $cdmunicipio = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NMBAIRROENDERECO':
                        $nmbairroendereco = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDLOGRADOURO':
                        $cdlogradouro = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DSENDERECO':
                        $dsendereco = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'NRIMOVELENDERECO':
                        $nrimovelendereco = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'DSCOMPLEENDERECO':
                        $dscompleendereco = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'DSREFERENCIAENDE':
                        $dsreferenciaende = $campo['DSVALORITEMSOLI']; 
                        break;
                }
            }
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));

            $endereco = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(array("nrorg"=> $nrorg,"nrparcnegocio" => $result->getNrparcnegocio(), "cdtipoendereco" => $cdtipoendereco));

            if(!$endereco) {
                $endereco = new $this->enderecoParc;
                $endereco->setNrenderecoparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ENDERECOPARC', 1, 12, true));
                $endereco->setNrparcnegocio($result->getNrparcnegocio());
                $endereco->setCdtipoendereco($cdtipoendereco);
                $endereco->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $endereco->setDtinclusao(DateUtil::getDataAtual(false));
                $endereco->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $endereco->setCdoperinclusao($this->hcmParameters->getCdoperador());
                
            }
            
            $endereco->setCdlogradouro($cdmunicipio);
            $endereco->setCdmunicipio($cdlogradouro);
            $endereco->setCdpais($cdpais);
            $endereco->setDsendereco($dsendereco);
            $endereco->setNmbairroendereco($nmbairroendereco);
            $endereco->setNrcependereco($nrcependereco);
            $endereco->setNrimovelendereco($nrimovelendereco);
            $endereco->setDscompleendereco($dscompleendereco);
            $endereco->setDsreferenciaende($dsreferenciaende);
            $endereco->setSgestado($sgestado);
           
            $this->entityManager->persist($endereco);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }    
    }
    
    public function validaAlteraEndereco($rowSolicitacao){
    try {
            $itensSolicitacao = $this->solicitacaoRepository->retornaItensSolicitacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRSOLICITACAO']);
             foreach ($itensSolicitacao as $campo) {
                if($campo['DSCHAVEITEMSOLI'] == 'CDTIPOENDERECO')
                    $cdtipoendereco = $campo['DSVALORITEMSOLI'];
            }
            
            $nrorg = $rowSolicitacao['NRORG'];
            $nrvinculom = $rowSolicitacao['NRVINCULOM'];
            
            
            if(!isset($cdtipoendereco))
                return false;
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));

            $endereco = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(array("nrorg"=> $nrorg,"nrparcnegocio" => $result->getNrparcnegocio(), "cdtipoendereco" => $cdtipoendereco));
            
            if($endereco)
                return false;
            else    
                return true;
                
        } catch (\Exception $e) {
            throw $e;
        }    
    }
    
    public function alteraComunicacao($nrorg,$nrvinculom,$itens){
        try {
            $cdprefixcomuparc = null;
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'CDFORMACOMU':
                        $cdformacomu = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDPREFIXCOMUPARC':
                        $cdprefixcomuparc = $campo['DSVALORITEMSOLI'];
                        break;
                    case 'CDCOMUNICAPARC':
                        $cdcomunicaparc = $campo['DSVALORITEMSOLI'];
                        break;
                }
            }
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));
            $comunica = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array("nrorg"=> $nrorg,"nrparcnegocio" => $result->getNrparcnegocio(), "cdformacomu" => $cdformacomu));
            
            if($comunica){
                if(!is_null($cdprefixcomuparc))
                    $comunica->setCdprefixcomuparc($cdprefixcomuparc);
                
                $comunica->setCdcomunicaparc($cdcomunicaparc);
            }
            else {
                $comunica = new $this->comunicaParc;
                $comunica->setNrcomunicaparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'COMUNICAPARC', 1, 12, true));
                $comunica->setNrparcnegocio($result->getNrparcnegocio());
                $comunica->setCdprefixcomuparc($cdprefixcomuparc);
                $comunica->setCdcomunicaparc($cdcomunicaparc);
                $comunica->setCdformacomu($cdformacomu);
                $comunica->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $comunica->setDtinclusao(DateUtil::getDataAtual(false));
                $comunica->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $comunica->setCdoperinclusao($this->hcmParameters->getCdoperador());
                
            }
           
            $this->entityManager->persist($comunica);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }    
    }
    
    public function validaAlteraComunicacao($rowSolicitacao){
        try {
            $itensSolicitacao = $this->solicitacaoRepository->retornaItensSolicitacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRSOLICITACAO']);
             foreach ($itensSolicitacao as $campo) {
                if($campo['DSCHAVEITEMSOLI'] == 'CDFORMACOMU')
                    $cdformacomu = $campo['DSVALORITEMSOLI'];
            }
            
            $nrorg = $rowSolicitacao['NRORG'];
            $nrvinculom = $rowSolicitacao['NRVINCULOM'];
            
            if(!isset($cdformacomu))
                return false;
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));
            $comunica = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array("nrorg"=> $nrorg,"nrparcnegocio" => $result->getNrparcnegocio(), "cdformacomu" => $cdformacomu));
            if($comunica)
                return false;
            else    
                return true;
                
        } catch (\Exception $e) {
            throw $e;
        }    
    }

    public function alteraConta($nrorg,$nrvinculom,$itens){
        try {
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                case 'CDBANCO':
                    $cdbanco = $campo['DSVALORITEMSOLI'];
                    break;
                case 'CDAGENCIA':
                    $cdagencia = $campo['DSVALORITEMSOLI'];
                    break;
                case 'CDCONTCORR':
                    $cdcontcorr = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NRTPCONTCORR':
                    $nrtpcontcorr = $campo['DSVALORITEMSOLI'];
                    break;
                }
            }
            $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $result->getNrpessoa()));

            $contcorr = $this->entityManager->getRepository(Repositories::CONTCORRPARC)->findOneBy(array("nrorg"=> $nrorg,"nrparcnegocio" => $result->getNrparcnegocio(),"nrtpcontcorr" => $nrtpcontcorr));
            
            if(!$contcorr){
                $contcorr = new $this->contaCorrenteParc;
                $contcorr->setNrcontcorrparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'CONTCORRPARC', 1, 12, true));
                $contcorr->setNrparcnegocio($result->getNrparcnegocio());
                $contcorr->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $contcorr->setDtinclusao(DateUtil::getDataAtual(false));
                $contcorr->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $contcorr->setCdoperinclusao($this->hcmParameters->getCdoperador());
            }
            $contcorr->setCdbanco($cdbanco);
            $contcorr->setCdagencia($cdagencia);
            $contcorr->setCdcontcorr($cdcontcorr);
            $contcorr->setNrtpcontcorr($nrtpcontcorr);
            $this->entityManager->persist($contcorr);
            $this->entityManager->flush();
            
        } catch (\Exception $e) {
            throw $e;
        }    
    }
    
    public function insereProcesso($nrorg,$nrvinculom,$cdstatus,$itens){
    try {
            $dataset = array();
            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                case 'NRTEMPLATE':
                    $dataset["NRTEMPLATE"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NRTIPOPROCESSO':
                    $dataset["NRTIPOPROCESSO"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NRPARCNEGOCIORESP':
                    $dataset["NRPARCNEGOCIORESP"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'DSMOTIVO':
                    $dataset["DSMOTIVO"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'LOCALIDADE':
                    $dataset["LOCALIDADE"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'DSDESCRICAO':
                    $dataset["DSDESCRICAO"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'DTSOLICITA':
                     $dataset["DTSOLICITA"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'DTLIMSOLICITA':
                     $dataset["DTLIMSOLICITA"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NROCUPACAOM':
                    $dataset["NROCUPACAOM"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NRESTRUTGEREN':
                    $dataset["NRESTRUTURAM"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NRVAGAS':
                    $dataset["NRVAGAS"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'IDPUBLICAVAGA':
                    $dataset["IDPUBLICAVAGA"] = $campo['DSVALORITEMSOLI'];
                    break;
                case 'NMPROCESSO':
                    $dataset["NMPROCESSO"] = $campo['DSVALORITEMSOLI'];
                    break;
                }
            }
            $dataset["update"] = false;
            $dataset["CDSTATUS"] = $cdstatus;
            $this->processosService->adicionarUpdateProcesso($dataset);
            
        } catch (\Exception $e) {
            throw $e;
        }    
    }

    public function alteraEscala($nrorg, $nrvinculom , $itens)
    {
        try {
            $nrturno = null;
            $nrmotivoescala = null;
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);

            foreach ($itens as $campo) {
                switch($campo['DSCHAVEITEMSOLI']){
                    case 'NRESCALATRABM':
                        $nrescalatrabm = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'NRTURNO':
                        $nrturno = $campo['DSVALORITEMSOLI']; 
                        break;
                    case 'DTINIESCALA':
                        $dtiniescala = DateUtil::getDataDeString(substr($campo['DSVALORITEMSOLI'],0,10), DateUtil::FORMATO_BRASILEIRO, true);
                        break;
                    case 'NRMOTIVOESCALA':
                        $nrmotivoescala = $campo['DSVALORITEMSOLI']; 
                        break;
                }
            }
            
            $result = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtiniescala" => $dtiniescala));

            if($result)
                throw new \Exception("Já existe uma alteração de escala para este vinculo nesta data");
            $this->solicitacaoRepository->atualizaAlteEscala($nrorg,$nrvinculom,DateUtil::subtraiIntervalo($dtiniescala,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),$this->hcmParameters->getCdoperador());
            
            $alteEscala = new $this->alteEscala;
            $alteEscala->setNrorg($nrorg);
            $alteEscala->setNrvinculom($nrvinculom);
            $alteEscala->setNrturno($nrturno);
            $alteEscala->setNrmotivoescala($nrmotivoescala);
            $alteEscala->setNrescalatrabm($nrescalatrabm);
            $alteEscala->setDtiniescala($dtiniescala);
            $alteEscala->setDtinclusao(DateUtil::getDataAtual(false));
            $alteEscala->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $alteEscala->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $alteEscala->setNralteescala(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true));
            $this->entityManager->persist($alteEscala);
            $this->entityManager->flush();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function realizaDemissao($nrorg, $nrvinculom , $itens)
    {
        try {
            $nrturno = null;
            $nrmotivoescala = null;
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
   		    $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
            $gpevinculoH = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
            $nrestruturalegal = $gpevinculoH->getNrestrutlegal();
            $nrtpavipre = null;
            $nrtpdemissao = null;
            $nrmotivoresc = null;
            $dtavisoprevio = null;
            $dtrescisaovinc = null;
             foreach ($itens as $campo) {
                if($campo['DSCHAVEITEMSOLI'] == 'NRTPAVIPRE')
                    $nrtpavipre = $campo['DSVALORITEMSOLI'];
                if($campo['DSCHAVEITEMSOLI'] == 'NRTPDEMISSAO')
                    $nrtpdemissao = $campo['DSVALORITEMSOLI'];
                if($campo['DSCHAVEITEMSOLI'] == 'NRMOTIVORESC')
                    $nrmotivoresc = $campo['DSVALORITEMSOLI'];
                if($campo['DSCHAVEITEMSOLI'] == 'DTAVISOPREVIO')
                    $dtavisoprevio = $campo['DSVALORITEMSOLI'];
                if($campo['DSCHAVEITEMSOLI'] == 'DTRESCISAOVINC')
                    $dtrescisaovinc = $campo['DSVALORITEMSOLI'];
            }
            if($gpevinculom->getDtrescisaovinc() && $dtrescisaovinc){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrestruturalegal, null, $dtrescisaovinc,$dtcompetencia->format('d/m/Y'), $acao, $nrtpdemissao);
            }else if(!$gpevinculom->getDtrescisaovinc() && $dtrescisaovinc){
                $acao = "INSERE";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrestruturalegal, null, $dtrescisaovinc,$dtcompetencia->format('d/m/Y'), $acao, $nrtpdemissao);
            }
            $this->alteSituacaoFuncionalService->manipulaSituacaoFuncional($dtrescisaovinc,$nrorg,$nrvinculom,$gpevinculom->getDtadmissaovinc(),$acao,null);
            
            $gpevinculom->setNrtpavipre($nrtpavipre);
            $gpevinculom->setNrtpdemissao($nrtpdemissao);
            $gpevinculom->setNrmotivoresc($nrmotivoresc);
            $gpevinculom->setDtavisoprevio(DateUtil::getDataDeString(substr($dtavisoprevio,0,10), DateUtil::FORMATO_BRASILEIRO, true));
            $gpevinculom->setDtrescisaovinc(DateUtil::getDataDeString(substr($dtrescisaovinc,0,10), DateUtil::FORMATO_BRASILEIRO, true));
            
            $this->entityManager->persist($gpevinculom);
            $this->entityManager->flush();

            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaDemissao($rowSolicitacao){
        try {
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
            
            $itensSolicitacao = $this->solicitacaoRepository->retornaItensSolicitacao($rowSolicitacao['NRORG'], $rowSolicitacao['NRSOLICITACAO']);
             foreach ($itensSolicitacao as $campo) {
                if($campo['DSCHAVEITEMSOLI'] == 'NRTPDEMISSAO')
                    $nrtpdemissao = $campo['DSVALORITEMSOLI'];
                if($campo['DSCHAVEITEMSOLI'] == 'DTRESCISAOVINC')
                    $dtrescisaovinc = $campo['DSVALORITEMSOLI'];
            }

            $nrorg = $rowSolicitacao['NRORG'];
            $nrvinculom = $rowSolicitacao['NRVINCULOM'];
            
            $mensagens = array();

            if(!$this->alteSituacaoFuncionalService->verificaEstabilidade($nrorg, $this->hcmParameters->getDtCompetencia(), $nrvinculom, $dtrescisaovinc));
                $mensagens[] = "Você está rescindindo um colaborador com uma situação de estabilidade.";
            if(!$this->vinculoServices->verificaDissidio($nrorg, $nrvinculom, $this->hcmParameters->getDtCompetencia(), $dtrescisaovinc, $nrtpdemissao));
                $mensagens[] = "Você está rescindindo um colaborador em uma data próxima ao seu dissídio.";
            if(!$this->alteSituacaoFuncionalService->verificaVinculoAfastado($nrorg, $nrvinculom, $dtrescisaovinc));
                $mensagens[] = "Você está rescindindo um colaborador em afastamento.";
            
            return $mensagens;
        } catch (\Exception $e) {
            throw $e;
        }    
    }


    public function realizaAdmissao($nrorg, $nrvinculom , $itens)
    {
        try {
             foreach ($itens as $campo) {
                if($campo['DSCHAVEITEMSOLI'] == 'DTMESCOMPETENC')
                    $competencia = $campo['DSVALORITEMSOLI'];
             }
             $this->solicitacaoRepository->atualizaCompetenciaVinculo($nrorg,$nrvinculom,$competencia);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function reprovaAdmissao($nrorg, $nrvinculom)
    {
        try {
            $vinc = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg" => $nrorg, "nrvinculom" => $nrvinculom));
            $vinc->setDtrescisaovinc($vinc->getDtadmissaovinc());
            $this->entityManager->persist($vinc);
            $this->entityManager->flush();
            $this->solicitacaoRepository->deletaHistoricoAdmissao($nrorg,$nrvinculom);


        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getOperadorRespInformal($nrvinculom, $nrorg, $dtmescompetenc){
        try{
            return $this->solicitacaoRepository->getOperadorRespInformal($nrvinculom, $nrorg, $dtmescompetenc);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
}  
